(("en" (("audio" ("This part of fluxus is responsible for capturing the incoming sound, and processing it into harmonic data, using fft (Fast Fourier Transform). The harmonics are bands of frequency which the sound is split into, giving some indication of the quality of the sound. It's the same as you see on a graphic equaliser - in fact, one of the example scripts (bars.scm) acts as a graphic equaliser display, and should be used to test the audio is working." " (start-audio \"alsa_pcm:capture_1\" 1024 44100)\n (define (animate)\n\t\t(colour (vector (gh 1) (gh 2) (gh 3))) ; make a colour from the harmonics, and set it to be the current colour \n\t\t(draw-cube)) ; draw a cube with this colour\n (every-frame (animate))\n" (("start-audio" ("jackport-string buffersize-number samplerate-number" "void" "Starts up the audio with the specified settings, you'll need to call this first, or put it into $HOME/.fluxus.scm to call it automatically at startup. Make the jack port name an empty string and it won't try to connect to anything for you. You can use qjackctrl or equivelent to do the connection manually. Fluxus reads a single mono source." " (start-audio \"alsa_pcm:capture_1\" 1024 44100)\n")) ("gh" ("harmonic-number" "harmonic-real" "Fluxus converts incoming audio into harmonic frequencies, which can then be plugged into your animations using this command. There are 16 harmonic bands availible, the harmonic-value argument will be wrapped around if greater or less than 16, so you can use this command without worrying about out of range errors." " (define (animate)\n\t\t(colour (vector (gh 1) (gh 2) (gh 3))) ; make a colour from the harmonics, and set it to be the current colour \n\t\t(draw-cube)) ; draw a cube with this colour\n (every-frame (animate))\n")) ("gain" ("gain-number" "void" "Sets the gain level for the fft sound, it's 1 by default." " (gain 100) ; too quiet?!\n")) ("process" ("wavfile-string" "void" "This command temporarally disables the realtime reading of the input audio stream and reads a wav file instead. For use with the framedump command to process audio offline to make music videos. The advantage of this is that it locks the framerate so the right amount of audio gets read for each frame - making syncing of the frames and audio files possible." " (process \"somemusic.wav\") ; read a precorded audio file\n")) ("smoothing-bias" ("value-number" "void" "A kind of weighted average for the harmonic bands which smooth them out over time. This setting defaults to 1.5. The best value really depends on the quality of the music, and the buffer sizes, and ranges from 0 -> 2. It's more obvious if you give it a try with the bars.scm script" " (smoothing-bias 0) ; no smoothing\n")) ("update-audio" ("" "void" "Updates the audio subsytem. This function is called for you (per frame) in fluxus-canvas.ss." " (smoothing-bias 0) ; no smoothing\n"))))) ("renderer" ("These commands are the low level renderer controls. You shouldn't need to deal with these unless you are being wily, or implementing a fluxus renderer outside of the scratchpad interface." "" (("make-renderer" ("" "rendererid-number" "Makes a new scenegraph renderer." " (make-renderer) \n")) ("renderer-grab" ("rendererid-number" "void" "Make this renderer the current context for commands." " (renderer-grab renderer) \n")) ("renderer-ungrab" ("" "void" "Pop the renderer context stack." " (renderer-grab renderer) \n")) ("fluxus-render" ("" "void" "Clears the backbuffer, and renders everything" " (fluxus-render) \n")) ("tick-physics" ("" "void" "Update the physics system." " (tick-physics) \n")) ("render-physics" ("" "void" "Render the physics system (for helper graphics)." " (render-physics) \n")) ("reset-renderers" ("" "void" "Deletes all the renderers and makes a new default one." " (reset-renderers) \n")) ("reshape" ("width-number height-number" "void" "Calls reshape on the current renderer" " (reshape 100 100) \n")) ("fluxus-init" ("" "void" "Inits the whole rendering system, only needs calling once." " (fluxus-init) \n")) ("fluxus-error-log" ("" "void" "Returns a string containing error information for the last frame." " (display (fluxus-error-log))\n"))))) ("global-state" ("Global state is really anything that controls the renderer globally, so it affects all primitives or controls the renderer directly - ie camera control or full screen effects like blurring." "" (("clear-engine" ("" "void" "Clears the renderer, and physics system. This command should not be called directly, use clear instead, as this clears a few other things, and calls clear-engine itself." " (clear-engine) ; woo hoo!\n")) ("blur" ("amount-number" "void" "Sets the full screen blur setting. Less is more, but if you set it too low it will make the on screen editing impossible to read, so save your script first :)" " (blur 0.1) ; for nice trails\n")) ("fog" ("fogcolour-vector amount-number begin-number end-number" "void" "Sets the fogging parameters to give a visual depth cue (aerial perspective in painter's jargon). This can obscure the on screen editing, so keep the amount small." " (clear-colour (vector 0 0 1))   ; looks nice if the background matches\n (fog (vector 0 0 1) 0.01 1 100) ; blue fog\n")) ("show-axis" ("show-number" "void" "Shows the worldspace origin axis used." " (show-axis 1)\n")) ("show-fps" ("show-number" "void" "Shows an fps count in the lower left of the screen. used." " (show-fps 1)\n")) ("lock-camera" ("primitiveid-number" "void" "Locks the camera transform onto the specified primitive's transform. It's like parenting the camera to the object. This is the easiest way to procedurally drive the camera. Use an id number of 0 to unlock the camera." " (clear)\n (define obj (build-cube)) ; make a cube for the camera to lock to\n \n (push) ; make a background cube so we can tell what's happening\n (hint-wire)  \n (hint-unlit) \n (colour (vector 0 0.4 0))\n (scale (vector -50 -50 -50))\n (build-cube)\n (pop)\n \n (lock-camera obj) ; lock the camera to our first cube\n \n (define (animate)\n     (grab obj)\n     (rotate (vector 1 0 0)) ; rotate the cube\n     (ungrab))\n \n (every-frame (animate))\n")) ("camera-lag" ("amount-number" "void" "The camera locking has an inbuilt lagging which means it will smoothly blend the movement relative to the primitive it's locked to." " (clear)\n (define obj (build-cube)) ; make a cube for the camera to lock to\n \n (push) ; make a background cube so we can tell what's happening\n (hint-wire)\n (hint-unlit)\n (colour (vector 0 0.4 0))\n (scale (vector -50 -50 -50))\n (build-cube)\n (pop)\n \n (lock-camera obj) ; lock the camera to our first cube\n (camera-lag 0.1)  ; set the lag amount, this will smooth out the cube jittery movement\n \n (define (animate)\n     (grab obj)\n     (identity)\n     (translate (vector (modulo (round (inexact->exact (time))) 6) 0 0)) ; make a jittery movement\n     (ungrab))\n \n (every-frame (animate))\n")) ("load-texture" ("pngfilename-string" "textureid-number" "Loads a texture from disk, converts it to a texture, and returns the id number. The texture loading is memory cached, so repeatedly calling this will not cause it to load again. Use force-load-texture if you are changing the texture while running the script. The png may be RGB or RGBA to use alpha transparency." " (texture (load-texture \"mytexture.png\"))\n (build-cube) ; the cube will be texture mapped with the image\n")) ("force-load-texture" ("pngfilename-string" "textureid-number" "Uncached loading of textures from disk, converts it to a texture, and returns the id number. Useful if you are changing the texture while running the script, otherwise use load-texture, which will be much faster. The png may be RGB or RGBA to use alpha transparency." " (texture (force-load-texture \"mytexture.png\"))\n (build-cube) ; the cube will be texture mapped with the image\n")) ("frustum" ("top-number bottom-number left-number right-number" "void" "Sets the camera frustum, and thus the aspect ratio of the frame." " (frustum -1 1 -0.75 0.75) ; default settings\n")) ("clip" ("front-number back-number" "void" "Sets the front & back clipping planes for the camera frustum, and thus the viewing angle. Change the front clipping distance to alter the perspective from telephoto to fisheye." " (clip 1 10000) ; default settings\n")) ("ortho" ("" "void" "Sets orthographic projection - i.e. no perspective." " (ortho) \n")) ("persp" ("" "void" "Sets perspective projection (the default) after ortho has been set." " (persp) \n")) ("set-ortho-zoom" ("amount-number" "void" "Sets the zoom level for the orthographic projection." " (set-ortho-zoom 2) \n")) ("backfacecull" ("setting-number" "void" "Turns backface culling on or off. Backface culling speeds up rendering by removing faces not orientated towards the camera. Defaults to on, but this is not always desired, eg for double sided polygons." " (backfacecull 0) \n")) ("clear-colour" ("colour-vector" "void" "Sets the colour we clear the renderer with, this forms the background colour for the scene." " (clear-colour (vector 1 0 0)) ; RED!!!\n")) ("clear-frame" ("setting-number" "void" "Sets the frame clearing on or off." " (clear-frame 0) \n")) ("clear-zbuffer" ("setting-number" "void" "Sets the zbuffer clearing on or off." " (clear-zbuffer 0) \n")) ("clear-accum" ("setting-number" "void" "Sets the accumulation buffer clearing on or off." " (clear-accum 1) \n")) ("get-camera" ("" "matrix-vector" "Gets the current camera transform matrix. This is the low level function, use get-camera-transform instead." " (get-camera) \n")) ("get-locked-matrix" ("" "matrix-vector" "Gets the current camera lock transform matrix. Takes the lag into account" " (get-locked-matrix) \n")) ("set-camera" ("" "void" "Sets the camera transform matrix. This is the low level interface used by set-camera-transform, which you should generally use instead." " (set-camera) \n")) ("get-projection-transform" ("" "projection-matrix" "Gets the current projection matrix." " (get-projection-transform) \n")) ("get-screen-size" ("" "size-vector" "Returns a vector containing the current width and height of the window." " (get-screen-size) \n")) ("set-screen-size" ("size-vector" "void" "Sets the window width and height." " (set-screen-size (vector 10 10)) ; small window time :) \n")) ("select" ("screenxpos-number screenypos-number pixelssize-number" "primitiveid-number" "Looks in the region specified and returns the id of the closest primitive to the camera rendered there, or 0 if none exist." " (display (select 10 10 2))(newline)\n")) ("desiredfps" ("fps-number" "void" "Throttles the renderer so as to not take 100% cpu. This gives an upper limit on the fps rate, which doesn't quite match the given number, but I'm working on it..." " (desiredfps 100000) ; makes fluxus render as fast as it can, and take 100% cpu.\n")) ("draw-buffer" ("buffer_name" "void" "Select which buffer to draw in for stereo mode you'd do 'back-right and 'back-left" " (draw-buffer 'back)\n")) ("read-buffer" ("buffer_name" "void" "Select which buffer to read from" " (read-buffer 'back)\n")) ("set-stereo-mode" ("mode" "bool" "select which stereo mode to use currently only 'crystal-eyes and 'no-stereo are supported the return indicates if the operation was successful or not 'crystal-eyes will return false if you don't have a stereo window" " (set-stereo-mode 'crystal-eyes)\n")) ("set-colour-mask" ("vector" "void" "sets the colour mask give it a quat of booleans which correspond to the red, green, blue and alpha channels respectively after this operation you'll only see those colour which you set to true (this is useful for stereo with red-blue glasses)" " (set-colour-mask #(#t #f #f #t))\n")) ("shadow-light" ("number-setting" "void" "Sets the light to use for generating shadows, set to 0 to disable shadow rendering." " (shadow-light 1) \n")) ("shadow-length" ("number-setting" "void" "Sets the length of the shadow volume rendering." " (shadow-length 10) \n")) ("shadow-debug" ("number-setting" "void" "Turns on debug rendering of the shadow volume rendering." " (shadow-debug 1) \n")) ("accum" ("mode-symbol value-number" "void" "Controls the accumulation buffer (just calls glAccum under the hood). Possible symbols are: accum load return add mult" " (accum 'add 1) \n"))))) ("lights" ("Without lights you wouldn't be able to see anything. Luckily fluxus gives you one for free by default, a white diffuse point light attached to the camera. For more interesting lighting, you'll need these functions. Using the standard fixed function graphics pipeline, simplistically speaking, OpenGL multiplies these values with the surface material (set with local state commands like ambient and diffuse) and the texture colour value to give the final colour." "" (("make-light" ("type-symbol cameralocked-symbol" "lightid-number" "Makes a new light. The type can be one of: point, directional or spot. If the cameralocked string is not free then it will be attached to the camera, and move around when you move the camera." " (make-light 'spot 'locked)\n")) ("light-ambient" ("lightid-number colour" "void" "Sets the ambient contribution for the specified light." " (light-ambient mylight (vector 1 1 1)) ; a boring light\n")) ("light-diffuse" ("lightid-number colour" "void" "Sets the diffuse contribution for the specified light." " (light-diffuse mylight (vector 1 1 1)) \n")) ("light-specular" ("lightid-number colour" "void" "Sets the specular contribution for the specified light." " (light-specular mylight (vector 1 1 1)) \n")) ("light-position" ("lightid-number position-vector" "void" "Sets the position of the specified light. In worldspace if free, in camera space is attached." " (light-position mylight (vector 0 100 0)) \n")) ("light-spot-angle" ("lightid-number angle-number" "void" "Sets the spotlight cone angle of the specified light. If it's not a spot light, this command has no effect." " (light-position mylight (vector 0 100 0)) \n")) ("light-spot-exponent" ("lightid-number exponent-number" "void" "Sets the spotlight exponent (fuzzyness of the cone) of the specified light. If it's not a spot light, this command has no effect." " (light-spot-exponent mylight 0.1) \n")) ("light-attenuation" ("lightid-number type-symbol attenuation-number" "void" "Sets the light attenuation (fade off with distance) of the specified light. The type symbol can be one of: constant, linear or quadratic." " (light-attenuation 'constant 0.1) \n")) ("light-direction" ("lightid-number direction-vector" "void" "Sets the direction of a directional light. If it's not a directional light, this command has no effect." " (light-spot-exponent mylight 0.1) \n"))))) ("local-state" ("The local state functions control rendering either for the current state - or the state of the currently grabbed primitive. In fluxus state means the way that things are displayed, either turning on and off rendering features, changing the style of different features, or altering the current transform." "" (("push" ("" "void" "Pushes a copy of the current drawing state to the top of the stack. The drawing state contains information about things like the current colour, transformation and hints." " (colour (vector 1 0 0)) ; set current colour to red\n (push)                  ; copy and push drawing state\n (colour (vector 0 1 0)) ; set current colour to green\n (draw-cube)             ; draws a green cube\n (pop)\t\t\t\t   ; forget old drawing state\n ; current colour is now red again\n")) ("pop" ("" "void" "Destroys the current drawing state, and sets the current one to be the previously pushed one in the stack. The drawing state contains information about things like the current colour, transformation and hints." " (colour (vector 1 0 0)) ; set current colour to red\n (push)                  ; copy and push drawing state\n (colour (vector 0 1 0)) ; set current colour to green\n (draw-cube)             ; draws a green cube\n (pop)                   ; forget old drawing state\n ; current colour is now red again\n")) ("grab" ("object-id" "void" "Grabs the specified object. Once an object has grabbed it's state can be modified using the same commands used to set the current drawing state. (ungrab) needs to be used to return to the normal drawing state. Grabbing can also be stacked, in which case ungrab pops to the last grabbed primitive." " (colour (vector 1 0 0))      ; set the current colour to red\n (define mycube (build-cube)) ; makes a red cube \n (grab mycube)  \t\t\t\t       \n (colour (vector 0 1 0)) ; sets the cubes colour to green \n (ungrab)\t\t\t\t   ; return to normal state\n")) ("ungrab" ("" "void" "Ungrabs the currently grabbed object, and either returns to the normal drawing state, or pops to the last grabbed primitive." " (colour (vector 1 0 0))      ; set the current colour to red\n (define mycube (build-cube)) ; makes a red cube \n (grab mycube)  \t\t\t\t       \n (colour (vector 0 1 0)) ; sets the cubes colour to green \n (ungrab)\t\t\t\t   ; return to normal state\n")) ("apply" ("object-id" "void" "Applies the current object transform to the vertex positions of the supplied object and sets it's transform to identity." " (rotate (vector 45 0 0))     \n (define mycube (build-cube)) ; makes a cube with a rotation \n (apply mycube)  \t\t\t\t; applies the rotation to the points of the cube\n")) ("opacity" ("value" "void" "Sets the opacity of the current drawing state, or the currently grabbed primitive." " (opacity 0.5)     \n (define mycube (build-cube)) ; makes a half transparent cube \n")) ("shinyness" ("value" "void" "Sets the shinyness of the current drawing state, or the currently grabbed primitive. This value sets the tightness of the specular highlight." " (shinyness 100)     \n (specular (vector 1 1 1)) ; sets the specular colour\n (define mysphere (build-sphere 10 10)) ; makes a shiny cube \n")) ("colour" ("colour-vector" "void" "Sets the colour of the current drawing state, or the currently grabbed primitive." " (colour (vector 1 0.5 0.1)) ; mmm orange...   \n (define mycube (build-cube)) ; makes an orange cube \n")) ("wire-colour" ("colour-vector" "void" "Sets the wire frame colour of the current drawing state, or the currently grabbed primitive. Visible with (hint-wire) on most primitives." " (wire-colour (vector 1 1 0)) ; set yellow as current wire colour\n (hint-wire)   \n (define mycube (build-cube)) ; makes a cube with yellow wireframe \n")) ("specular" ("colour-vector" "void" "Sets the specular colour of the current drawing state, or the currently grabbed primitive." " (specular (vector 0 0 1)) ; set blue as specular colour\n (define mysphere (build-sphere 10 10)) ; makes a shiny blue sphere\n")) ("ambient" ("colour-vector" "void" "Sets the ambient colour of the current drawing state, or the currently grabbed primitive." " (ambient (vector 0 0 1)) ; set blue as ambient colour\n (define mysphere (build-sphere 10 10)) ; makes a boringly blue sphere \n")) ("opacity" ("value" "void" "Sets the emissive colour of the current drawing state, or the currently grabbed primitive." " (emissive (vector 0 0 1)) ; set blue as emissive colour\n (define mysphere (build-sphere 10 10)) ; makes an bright blue sphere \n")) ("identity" ("" "void" "Sets the drawing state transform to identity, on the state stack, or the currently grabbed primitive." " (push)\n (scale (vector 2 2 2)) ; set the current scale to double in each dimension\n (define mycube (build-cube)) ; make a scaled cube \n (pop)\n (grab mycube)\n (identity) ; erases the transform and puts the cube back to its original state\n (ungrab)\n")) ("concat" ("matrix" "void" "Concatenates (multiplies) a matrix on to the current drawing state or grabbed primitive." " (define mymatrix (mrotate (vector 0 45 0))) ; make a matrix\n (concat mymatrix) ; concat it into the current state\n (build-cube) ; make a cube with this rotation\n")) ("translate" ("vector" "void" "Applies a translation to the current drawing state transform or grabbed primitive." " (transform (vector 0 1.4 0)) ; translates the current transform up a bit\n (build-cube) ; build a cube with this transform\n")) ("rotate" ("vector-or-quaternion" "void" "Applies a rotation to the current drawing state transform or grabbed primitive." " (rotate (vector 0 45 0)) ; turns 45 degrees in the Y axis\n (build-cube) ; build a cube with this transform\n")) ("scale" ("vector" "void" "Applies a scale to the current drawing state transform or grabbed primitive." " (scale (vector 0.5 0.5 0.5)) ; scales the current transform to half the size\n (build-cube) ; build a cube with this transform\n")) ("get-transform" ("" "matrix-vector" "Returns a matrix representing the current state transform or for the grabbed primitive." " (translate (vector 1 0 0))\n (display (get-transform))(newline) ; prints the current transform\n (define shape (build-sphere 10 10))\n (grab shape)\n (translate (vector 0 1 0))\n (display (get-transform))(newline) ; prints shape's transform\n (ungrab)\n")) ("parent" ("primitive-id" "void" "Parents the currently grabbed primitive to the supplied parent primitive. The current primitive will now be moved around with the parent by aquiring all the parent's transforms." " (define parent-prim (build-cube)) ; make a parent cube\n (translate (vector 2 0 0)) ; move a bit in x\n (parent parent-prim) ; set parent-prim as the current parent\n (define child-prim (build-cube)) ; make a child cube\n (grab parent-prim) \n (rotate (vector 0 45 0)) ; the child will now be moved by this transform in addition to its own\n (ungrab)\n")) ("line-width" ("value" "void" "Sets the line width (in screen space) of the current drawing state, or the currently grabbed primitive. Affects wireframe and things like that." " (line-width 5)\n (hint-wire)\n (build-sphere 10 10) ; make a sphere with thick wireframe\n")) ("point-width" ("value" "void" "Sets the point width (in screen space) of the current drawing state, or the currently grabbed primitive. Affects point rendering and particles in hardware point mode." " (point-width 5)\n (hint-points)\n (build-sphere 10 10) ; make a sphere with thick points\n")) ("blend-mode" ("src dst" "void" "Sets the blend mode of the current drawing state, or the currently grabbed primitive. This is the way that alpha is composited to the rendering surface. Blendmode symbols can consist of: zero one dst-color one-minus-dst-color src-alpha one-minus-src-alpha dst-alpha one-minus-dst-alpha Also src-alpha-saturate as an option for the source blendmode only." " ; list out all the possible blendmodes\n \n (define src-blend (vector 'zero 'one 'dst-color 'one-minus-dst-color 'src-alpha\n                     'one-minus-src-alpha 'dst-alpha 'one-minus-dst-alpha\n                     'src-alpha-saturate))\n \n (define dst-blend (vector 'zero 'one 'dst-color 'one-minus-dst-color 'src-alpha\n                     'one-minus-src-alpha 'dst-alpha 'one-minus-dst-alpha))\n \n ; picks a random element\n (define (pick-rnd-item l)\n     (vector-ref l (random (vector-length l))))\n \n ; make lots of random spheres\n (define (rnd-sphere n)\n     (push)\n     (hint-depth-sort)\n     (opacity 0.5)\n     (colour (vector (flxrnd) (flxrnd) (flxrnd)))\n \n     ; set a random blendmode\n     (blend-mode (pick-rnd-item src-blend) (pick-rnd-item dst-blend))\n \n     (translate (vector (flxrnd) (flxrnd) (flxrnd)))\n     (scale (vector 0.1 0.1 0.1))\n     (build-sphere 10 10)\n     (pop)\n     (if (zero? n)\n         0\n         (rnd-sphere (- n 1))))\n \n (clear)\n (clear-colour (vector 0.5 0.5 0.5))\n (rnd-sphere 100)\n")) ("hint-solid" ("" "void" "Sets the render hints to solid of the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-solid) ; this is the default render style so this isn't too exciting\n (build-cube) ; make a solid rendered cube \n")) ("hint-wire" ("" "void" "Sets the render hints to wireframe of the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-wire)\n (build-cube) ; make a wirefame rendered cube \n")) ("hint-normal" ("" "void" "Sets the render hints to display normals in the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-normal)\n (build-cube) ; display the normals on this cube \n")) ("hint-points" ("" "void" "Sets the render hints to display points in the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-points)\n (build-cube) ; display the vertex points on this cube \n")) ("hint-anti-alias" ("" "void" "Sets the render hints to anti-alias in the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-anti-alias)\n (build-cube) ; display a smoothed cube \n")) ("hint-unlit" ("" "void" "Sets the render hints to unlit in the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-unlit)\n (build-cube) ; display an unlit cube\n")) ("hint-vertcols" ("" "void" "Sets the render hints to use vertex colours in the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint. Vertex colours override the current (colour) state." "\n TODO:example doesnt seem to work\n (hint-vertcols)\n (define mycube (build-cube)) ; make a cube with vertcols enabled\n (grab mycube)\n (pdata-set \"c\" 0 (vector 0 1 0)) ; set the colour of the first vertex to green\n (ungrab)\n")) ("hint-box" ("" "void" "Sets the render hints to bounding box display in the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-box)\n (build-sphere 10 10) ; make a sphere with bounding box displayed\n")) ("hint-multitex" ("" "void" "Sets the render hints to use multitexturing in the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-multitexture)\n (multitexture 0 (load-texture \"tex1.png\"))\n (multitexture 1 (load-texture \"tex2.png\"))\n (build-sphere 10 10) ; make a sphere with overlayed textures\n")) ("hint-none" ("" "void" "Clears the render hints in the current drawing state, or the currently grabbed primitive. This allows you mainly to get rid of the default solid style, but also means that you can turn on and off hints without using push or pop." " (hint-none)\n (hint-wire)\n (build-cube) ; make a cube only visible with wireframe\n")) ("hint-origin" ("" "void" "Sets the render hints to display the object space origin of the primitive in the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-origin)\n (build-sphere 10 10) ; make a sphere with the origin displayed\n")) ("hint-cast-shadow" ("" "void" "(note: Not yet implemented) Sets the render hints to cast shadows for the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-origin)\n (build-sphere 10 10) ; make a sphere with the origin displayed\n")) ("hint-depth-sort" ("" "void" "Sets the render hints to depth sort for the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-depth-sort)\n (build-sphere 10 10) \n")) ("hint-ignore-depth" ("" "void" "Sets the render hints to ignore depth tests for the current drawing state, or the currently grabbed primitive. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint. This feature is useful for rendering transparent objects, as it means objects will be shown behind previously rendered ones." " TODO:) examples\n")) ("hint-lazy-parent" ("" "void" "Sets the render hints to prevent this primitive passing it's transform to it's children. Render hints change the way that primitives are rendered, but may have different effects - or no effect on certain primitive types, hence the name hint." " (hint-lazy-parent)\n (build-sphere 10 10) ; make a sphere with the origin displayed\n")) ("texture" ("textureid-number" "void" "Sets the texture of the current drawing state, or the currently grabbed primitive. Texture ids can be generated by the load-texture function." " (texture (load-texture \"mytexture.png\"))\n (build-sphere 10 10) ; make a sphere textured with mytexture.png\n")) ("multitexture" ("textureunit-number textureid-number" "void" "Sets the texture of the current drawing state, or the currently grabbed primitive in the same way as the texture function, but allows you to specify the texture unit (0-7) to apply the texture to. Multitexturing allows you to apply different textures and texture coordinates to the same object at once. Texture unit 0 is the default one (which uses the pdata \"t\" for it's texture coords) texture unit n looks for pdata \"tn\" - ie multitexture 1 looks for \"t1\". You need to add these yourself using (pdata-add) or (pdata-copy). Multitexturing is useful when the textures contain alpha, as they can be overlayed, i.e. decals placed on background textures. Note: fluxus needs to be built using scons MULTITEXTURE=1 to enable this feature." " (define obj (build-sphere 10 10)) ; make a sphere \n (grab obj)\n (multitexture 0 (load-texture \"mytextureA.png\")) \n (multitexture 1 (load-texture \"mytextureB.png\"))\n (pdata-add \"t1\" \"v\")   ; make some texture coords for texture B\n (pdata-copy \"t\" \"t1\")  ; copy them from the default texture coords\n (ungrab)\n")) ("print-scene-graph" ("" "void" "Prints out the current scene graph, useful for debugging." " (print-scene-graph) ; exciting...\n")) ("hide" ("hidden-number" "void" "Sets the hidden state for the grabbed primitive (also affects all child primitives). Hidden primitives can be treated as normal in every way - they just won't be rendered." " (define obj (build-cube))\n (grab obj)\n (hide 1) ; hide this cube\n (ungrab)\n")) ("selectable" ("selectable-number" "void" "Sets whether the grabbed primitive can be selected or not using the select command." " (define obj (build-cube))\n (grab obj)\n (selectable 0) ; now it won't be \"seen\" by calling select\n (ungrab)\n")) ("shader" ("vertexprogram-string fragmentprogram-string" "void" "Loads, compiles and sets the GLSL harware shader pair for the current drawing state, or the currently grabbed primitive. Requires OpenGL 2 support. The shader's uniform data can be controlled via shader-set! and all the pdata is sent through as per-vertex attribute data to the shader." " (push)\n ; assign the shaders to the surface\n (shader \"simplevert.glsl\" \"simplefrag.glsl\")\n (define s (build-sphere 20 20))\n (pop)\n \n (grab s)\n ; add and set the pdata - this is then picked up in the vertex shader \n ; as an input attribute called \"testcol\"\n (pdata-add \"testcol\" \"v\")\n (set-cols (pdata-size))\n (ungrab)\n \n (define (animate)\n     (grab s)\n \t; animate the deformamount uniform input parameter \n     (shader-set! (list \"deformamount\" (cos (time))))\n     (ungrab))\n \n (every-frame (animate))\n")) ("shader-set!" ("argument-list" "void" "Sets the uniform shader parameters for the GLSL shader. The list consists of token-string value pairs, which relate to the corresponding shader parameters names and values." " (push)\n ; assign the shaders to the surface\n (shader \"simplevert.glsl\" \"simplefrag.glsl\")\n (define s (build-sphere 20 20))\n (pop)\n \n (grab s)\n ; add and set the pdata - this is then picked up in the vertex shader \n ; as an input attribute called \"testcol\"\n (pdata-add \"testcol\" \"v\")\n (set-cols (pdata-size))\n (ungrab)\n \n (define (animate)\n     (grab s)\n \t; animate the deformamount uniform input parameter \n     (shader-set! (list \"deformamount\" (cos (time))))\n     (ungrab))\n \n (every-frame (animate))\n"))))) ("maths" ("These functions are optimised for 3D graphics, and the collision of computer science and maths is apparent here, so scheme vectors representing maths vectors are in this context taken to be 3 elements long, quaternions are vectors of length 4, and matrices are vectors of 16 elements long." "" (("vmul" ("vector number" "result-vector" "Multiplies a vector by a number" " (vmul (vector 1 2 3) 2)\n")) ("vadd" ("vector vector" "result-vector" "Adds two vectors together" " (vadd (vector 1 2 3) (vector 1 2 3))\n")) ("vsub" ("vector vector" "result-vector" "Subtracts a vector from another" " (vsub (vector 1 2 3) (vector 1 2 3))\n")) ("vdiv" ("vector number" "result-vector" "Divides a vector by a number" " (vdiv (vector 1 2 3) 2)\n")) ("vtransform" ("vector matrix" "result-vector" "Multiplies (transforms) a vector by a matrix" " (vtransform (vector 0 1 0) (mrotate (vector 90 0 0)))\n")) ("vtransform-rot" ("vector matrix" "result-vector" "Multiplies (transforms) a vector by a matrix, but leaves out the translation part. For operations involving normals." " (vtransform-rot (vector 0 1 0) (mrotate (vector 90 0 0)))\n")) ("vnormalise" ("vector" "result-vector" "Returns the normalised form of the vector (length=1)" " (vtransform-rot (vector 0 1 0) (mrotate (vector 90 0 0)))\n")) ("vdot" ("vector vector" "result-number" "Returns the dot product of two vectors" " (vdot (vector 0 1 0) (vector 1 0 0))\n")) ("vmag" ("vector" "result-number" "Returns the magnitude, or length of the vector" " (vmag (vector 0 1 1))\n")) ("vdist" ("vector vector" "result-number" "Treating the vectors as points, returns the distance between them." " (vdist (vector 100 100 0) (vector 0 0 100))\n")) ("vcross" ("vector vector" "result-vector" "Returns the cross product of two vectors, resulting in a vector that is perpendicular to the crossed ones." " (vcross (vector 100 100 0) (vector 0 0 100))\n")) ("mmul" ("matrix-vector matrix-vector" "matrix-vector" "Multiplies two matrices together" " (vmul (mtranslate (vector 1 0 0)) (mrotate (vector 0 90 0)))\n")) ("madd" ("matrix-vector matrix-vector" "matrix-vector" "Adds two matrices together" " (vadd (mtranslate (vector 1 0 0)) (mrotate (vector 0 90 0)))\n")) ("msub" ("matrix-vector matrix-vector" "matrix-vector" "Subtracts a matrix from another." " (vsub (mtranslate (vector 1 0 0)) (mrotate (vector 0 90 0)))\n")) ("mdiv" ("matrix-vector matrix-vector" "matrix-vector" "Divides a matrix by another" " (vdiv (mtranslate (vector 1 0 0)) (mrotate (vector 0 90 0)))\n")) ("mident" ("" "matrix-vector" "Returns the identity matrix" " (mident)\n")) ("mtranslate" ("vector" "matrix-vector" "Returns a matrix representing the specified transform" " (mtransform (vector 100 0 0))\n")) ("mrotate" ("vector" "matrix-vector" "Returns a matrix representing the specified rotation. Accepts a vector of euler angles, or a quaternion." " (mrotate (vector 0 45 0))\n")) ("mscale" ("vector" "matrix-vector" "Returns a matrix representing the specified scaling." " (mscale (vector 0.5 2 0.5))\n")) ("mtranspose" ("matrix-vector" "matrix-vector" "Returns the transpose of the input vector" " (mtranspose (mident))\n")) ("minverse" ("matrix-vector" "matrix-vector" "Returns the inverse of the input vector." " (minverse (mscale (vector 0.5 2 0.5)))\n")) ("maim" ("aim-vector up-vector" "matrix-vector" "Returns a matrix representing an aiming rotation so that the x axis points down the aim direction, and the y axis points up the up vector. Probably suffers from gimbal lock." " (maim (vector 0 0 1) (vector 0 1 0))\n")) ("qaxisangle" ("axis-vector angle-number" "quaternion-vector" "Returns the quaternion representing rotation of angle degrees about the specified axis." " (qaxisangle (vector 0 1 0) 45)\n")) ("qmul" ("quaternion-vector quaternion-vector" "quaternion-vector" "Multiplies two quaternions together." " (qmul (qaxisangle (vector 0 1 0) 45) (qaxisangle (vector 0 0 1) 180))\n")) ("qnormalise" ("quaternion-vector" "quaternion-vector" "Normalises a quaternion." " (qnormalise (qaxisangle (vector 0 19 0) 45))\n")) ("qtomatrix" ("quaternion-vector" "matrix-vector" "Converts a quaternion into a rotation matrix." " (qtomatrix (qaxisangle (vector 0 1 0) 45))\n")) ("qconjugate" ("quaternion-vector" "quaternion-vector" "Conjugatea a quaternion." " (qconjugate (qaxisangle (vector 0 1 0) 45))\n"))))) ("primitive-data" ("Primitive data (pdata for short) is fluxus' name for data which comprises primitives. In polygon primitives this means the vertex information, in particle primitives it corresponds to the particle information, in NURBS primitives it's the control vertices. Access to pdata gives you the ability to use primitives which are otherwise not very interesting, and deform and shape other primitives to give much more detailed models and animations. You can also add your own pdata, which is treated exactly like the built in types. Primitive data is named by type strings, the names of which depend on the sort of primitive. All pdata commands operate on the currently grabbed primitive." " ; a function to deform the points of an object\n (define (deform n)\n     (pdata-set! \"p\" n (vadd  (pdata-ref \"p\" n)                ; the original point, plus\n         (vmul (vector (flxrnd) (flxrnd) (flxrnd)) 0.1)))     ; a small random vector\n     (if (zero? n)\n         0\n         (deform (- n 1))))\n     \n (hint-unlit) ; set some render settings to\n (hint-wire)  ; make things easier to see\n (line-width 4)\n (define myobj (build-sphere 10 10)) ; make a sphere\n (grab myobj)\n (deform (pdata-size)) ; deform it\n (ungrab)\n" (("pdata-ref" ("type-string index-number" "value-vector/colour/matrix/number" "Returns the corresponding pdata element." " (pdata-ref \"p\" 1)\n")) ("pdata-set!" ("type-string index-number value-vector/colour/matrix/number" "void" "Writes to the corresponding pdata element." " (pdata-set! \"p\" 1 (vector 0 100 0))\n")) ("pdata-add" ("type-string name-string" "void" "Adds a new user pdata array. Type is one of \"v\":vector, \"c\":colour, \"f\":float or \"m\":matrix." " (pdata-add \"v\" \"mydata\")\n (pdata-set \"mydata\" 0 (vector 1 2 3))\n")) ("pdata-op" ("funcname-string pdataname-string operator" "void" "This is an experimental feature allowing you to do operations on pdata very quickly, for instance adding element for element one array of pdata to another. You can implement this in Scheme as a loop over each element, but this is slow as the interpreter is doing all the work. It's much faster if you can use a pdata-op as the same operation will only be one Scheme call." " (pdata-op \"+\" \"mydata\" (vector 1 2 3))  add a vector to all the pdata vectors\n (pdata-op \"+\" \"mydata\" \"myotherdata\")  add two pdata vectors element for element\n (pdata-op \"*\" \"mydata\" (vector 1 2 3))  multiply a vector to all the pdata vectors\n (pdata-op \"*\" \"mydata\" \"myotherdata\")  multiply two pdata vectors element for element\n (pdata-op \"closest\" \"p\" (vector 100 0 0))  returns position of the closest vertex to this point\n (pdata-op \"sin\" \"mydata\" \"myotherdata\")  sine of one float pdata to another\n (pdata-op \"cos\" \"mydata\" \"myotherdata\")  cosine of one float pdata to another\n")) ("pdata-copy" ("pdatafrom-string pdatato-string" "void" "Copies the contents of one pdata array to another. Arrays must match types." " (pdata-copy \"p\" \"mydata\") ; copy the vertex positions to a user array\n")) ("pdata-size" ("" "count-number" "Returns the size of the pdata arrays (they must all be the same). This is mainly used for iterating over the arrays." " (define (mashup n)\n     (pdata-set \"p\" n (vector (flxrnd) (flxrnd) (flxrnd))) ; randomise the vertex position\n     (if (zero? n)\n         0\n         (mashup (- n 1)))) ; loops till n is 0\n\n (define shape (build-sphere 10 10))\n (grab shape)\n (mashup (pdata-size)) ; randomise verts on currently grabbed primitive\n (ungrab)\n")) ("finalise" ("" "void" "Doesn't do anything anymore, I need to remove this :)" "")) ("recalc-normals" ("smoothornot-number" "void" "For polygon primitives only. Looks at the vertex positions and calculates the lighting normals for you automatically. Call with \"1\" for smooth normals, \"0\" for faceted normals." " (define shape (build-sphere 10 10)) ; build a sphere (which is smooth by default)\n (grab shape)\n (recalc-normals 0) ; make the sphere faceted\n (ungrab)\n"))))) ("physics" ("The physics system used in fluxus is based on the ode library, which allows you to add physical properties to objects and set them in motion. Since ODE is designed for rigid-body simulations, structures are described in terms of objects, joints and forces. A much more comprehensive explanation of these concepts can be found in the ODE documentation, which you have probably downloaded if you have compiled fluxus, or can be found at @url{http://ode.org/ode-docs.html} To help with debugging joints, try calling (render-physics) every frame, which will render locators showing you positions and axes of joints that have positional information." "" (("collisions" ("on/off-number" "void" "Enables or disables collision detection. Defaults to off." " (collisions 1)\n")) ("ground-plane" ("plane-vector offset-number" "void" "Create an infinite passive plane for use as the 'ground'" " (ground-plane (vector 0 1 0) 0)\n")) ("active-box" ("primitiveid-number" "void" "Enable the object to be acted upon by the physics system, using a box as the bounding volume. As an active object, it will be transformed by ode. Note: rotations only work correctly if you specify your transforms scale first, then rotate (translate doesn't matter) basically, ode can't deal with shearing transforms." " (define mycube (build-cube))\n (active-box mycube)\n")) ("active-cylinder" ("primitiveid-number" "void" "Enable the object to be acted upon by the physics system, using a cylinder as the bounding volume. As an active object, it will be transformed by ode. Note: rotations only work correctly if you specify your transforms scale first, then rotate (translate doesn't matter) basically, ode can't deal with shearing transforms." " (define mycube (build-cube))\n (active-cylinder mycube)\n")) ("active-sphere" ("primitiveid-number" "void" "Enable the object to be acted upon by the physics system, using a sphere as the bounding volume. As an active object, it will be transformed by ode. Note: rotations only work correctly if you specify your transforms scale first, then rotate (translate doesn't matter) basically, ode can't deal with shearing transforms." " (define mycube (build-cube))\n (active-sphere mycube)\n")) ("passive-box" ("primitiveid-number" "void" "Enable the object to be acted upon by the physics system, using a box as the bounding volume. As a passive object, active objects will collide with it, but it will not be transformed. Note: rotations only work correctly if you specify your transforms scale first, then rotate (translate doesn't matter) basically, ode can't deal with shearing transforms." " (define mycube (build-cube))\n (passive-box mycube)\n")) ("passive-cylinder" ("primitiveid-number" "void" "Enable the object to be acted upon by the physics system, using a cylinder as the bounding volume. As a passive object, active objects will collide with it, but it will not be transformed. Note: rotations only work correctly if you specify your transforms scale first, then rotate (translate doesn't matter) basically, ode can't deal with shearing transforms." " (define mycube (build-cube))\n (passive-cylinder mycube)\n")) ("passive-sphere" ("primitiveid-number" "void" "Enable the object to be acted upon by the physics system, using a sphere as the bounding volume. As a passive object, active objects will collide with it, but it will not be transformed. Note: rotations only work correctly if you specify your transforms scale first, then rotate (translate doesn't matter) basically, ode can't deal with shearing transforms." " (define mycube (build-cube))\n (passive-sphere mycube)\n")) ("surface-params" ("slip1-number slip2-number softerp-number softcfm-number" "void" "Sets some global surface attributes that affect friction and bouncyness. see section 7.3.7 of the ODE docs for an explanation of these parameters" " (surface-params 0.1 0.1 0.1 0.1)\n")) ("build-balljoint" ("primitiveid-number primitiveid-number axis-vector" "void" "Creates a balljoint to connect two objects (see the ode docs for a detailed description of the differences between the joint types). ODE considers joints to be a constraint that is enforced between two objects. When creating a joint, it is important to have the two primitives being joined in the desired positions before creating the joint. Joints can be created, modified and indexed in a similar way to other primitives." " (build-balljoint shape1 shape2 (vector 0 1 0)) \n")) ("build-fixedjoint" ("primitiveid-number" "void" "Creates a joint to connect an object to the global environment. This locks the object in place." " (build-fixedjoint shape) \n")) ("build-hingejoint" ("primitiveid1-number primitiveid2-number anchor-vector axis-vector" "hingeid-number" "Creates a ball joint to connect two objects (see the ode docs for a detailed description of the differences between the joint types). ODE considers joints to be a constraint that is enforced between two objects. When creating a joint, it is important to have the two primitives being joined in the desired positions before creating the joint. Joints can be created, modified and indexed in a similar way to other primitives." " (build-hingejoint shape1 shape2 (vector 0 1 0) (vector 0 1 0)) \n")) ("build-sliderjoint" ("primitiveid1-number primitiveid2-number axis-vector" "hingeid-number" "Creates a slider joint to connect two objects (see the ode docs for a detailed description of the differences between the joint types). ODE considers joints to be a constraint that is enforced between two objects. When creating a joint, it is important to have the two primitives being joined in the desired positions before creating the joint. Joints can be created, modified and indexed in a similar way to other primitives." " (build-sliderjoint shape1 shape2 (vector 0 1 0)) \n")) ("build-hinge2joint" ("primitiveid1-number primitiveid2-number anchor-vector axis1-vector axis2-vector" "hingeid-number" "Creates a hinge2 joint to connect two objects (see the ode docs for a detailed description of the differences between the joint types). ODE considers joints to be a constraint that is enforced between two objects. When creating a joint, it is important to have the two primitives being joined in the desired positions before creating the joint. Joints can be created, modified and indexed in a similar way to other primitives." " (build-hinge2joint shape1 shape2 (vector 0 100 0) (vector 0 1 0) (vector 0 1 0))\n")) ("build-amotorjoint" ("primitiveid1-number primitiveid2-number axis-vector" "hingeid-number" "Creates a angular motor joint to connect two objects (see the ode docs for a detailed description of the differences between the joint types). ODE considers joints to be a constraint that is enforced between two objects. When creating a joint, it is important to have the two primitives being joined in the desired positions before creating the joint. Joints can be created, modified and indexed in a similar way to other primitives." " (build-amotorjoint shape1 shape2 (vector 0 1 0))\n")) ("joint-param" ("jointid-number param-string value-number" "hingeid-number" "Sets the joint parameter for a joint where param is one of the following: \"HiStop\", \"Vel\", \"FMax\", \"FudgeFactor\", \"Bounce\", \"CFM\", \"StopERP\", \"StopCFM\",\"SuspensionERP\", \"SuspensionCFM\", \"Vel2\", \"FMax2\". see section 7.5.1 of the ODE docs for an explanation of each of these parameters, and which joint types they apply to." " (joint-param joint \"Vel\" 0.1)\n")) ("joint-angle" ("jointid-number angle-number vel-number" "void" "Set a new angle for this joint, with a given velocity taken to get there" " (joint-angle joint 90 0.1)\n")) ("set-max-physical" ("max-number" "void" "Sets the maximum number of objects the physics system can deal with. When the max level has been reached the oldest objects are automatically destroyed." " (set-max-physical 100)\n")) ("set-mass" ("primitiveid-number mass-number" "void" "Sets the mass of an active object" " (set-mass myshape 100)\n")) ("gravity" ("gravity-vector" "void" "Sets the strength and direction of gravity." " (gravity (vector 0 -1 0))\n")) ("kick" ("primitiveid-number kick-vector" "void" "Applies translation force to the object" " (kick myshape (vector 0 1 0))\n")) ("twist" ("primitiveid-number spin-vector" "void" "Applies rotational force to the object" " (twist myshape (vector 2 0 0))\n")) ("has-collided" ("primitiveid-number" "void" "Returns true if the grabbed object collided in the last frame" " (if (has-collided myshape) (display \"bang!\"))\n"))))) ("primitives" ("Primitives are objects that you can render. There isn't really much else in a fluxus scene, except lights, a camera and lots of primitives." "\n" (("build-cube" ("" "primitiveid-number" "A simple cube, texture mapped placement per face. @image{images/cube}" " (define mynewcube (build-cube))\n")) ("build-polygons" ("verts-number type-symbol" "primitiveid-number" "Builds a raw polygon primitive with size vertices (everything is initially set to zero). Type is a symbol that refers to the way the vertices are interpreted to build polygons, and can be one of the following: triangle-strip quad-list triangle-list triangle-fan polygon" " (define mynewshape (build-polygons 100 'triangle-strip))\n")) ("build-sphere" ("slices-number stacks-number" "primitiveid-number" "A sphere with the resolution specified, texture mapped in normal \"world map\" style. @image{images/sphere}" " (define mynewshape (build-sphere 10 10))\n")) ("build-plane" ("" "primitiveid-number" "A single quad plane, texture mapped from 0->1 in both dimensions. @image{images/plane}" " (define mynewshape (build-plane))\n")) ("build-seg-plane" ("vertsx-number vertsy-number" "primitiveid-number" "A tesselated poly plane, texture mapped from 0->1 in both dimensions." " (define mynewshape (build-plane))\n")) ("build-cylinder" ("hsegments rsegments" "primitiveid-number" "A capped cylinder, texture map wrapped around, and badly wrapped around the ends. @image{images/cylinder}" " (define mynewshape (build-cylinder 10 10))\n")) ("build-line" ("numpoints-number" "primitiveid-number" "Builds a line consisting of numpoints points. The geometry is constantly camera facing and is texture mapped so the texture is stretched along the line from start to finish. You use the pdata functions to edit the postions and widths of the lines. If used lit, the normals are faked to approximate a circular cross section. Additionally, if solid rendering is cleared with (hint-none) and (hint-wire) is activated, a faster constant width line will be drawn - width specified by the (line-width) command. @image{images/line}" " (define mynewshape (build-line 10))\n")) ("build-text" ("text-string" "primitiveid-number" "Builds a sequence of planes, texture mapped so that a font texture can be used to display text. Might also be useful for more abstract things. The font assumed to be non proportional - there is an example font shipped with fluxus @image{images/text} Ok, so this isn't a very good font texture :)" " (texture (texture-load \"font.png\"))\n (define mynewshape (build-text \"hello\"))\n")) ("build-nurbs-sphere" ("hsegments rsegments" "primitiveid-number" "Builds a tesselated nurbs sphere, texture mapped in the same fashion as the poly sphere. @image{images/nurbs-sphere}" " (define mynewshape (build-nurbs-sphere 10 10))\n")) ("build-nurbs-plane" ("hsegments rsegments" "primitiveid-number" "Builds a tesselated nurbs plane, texture mapped in uv direction. @image{images/nurbs-plane}" " (define mynewshape (build-nurbs-plane 10 10))\n")) ("build-particles" ("count-number" "primitiveid-number" "Builds a particles primitive containing num points, all initially set to the origin. You use the pdata functions to edit the postions, colours and sizes. Particles come in two flavors, camera facing sprites, which are the default, can be textured and individually scaled; and points (when hint-points is set), which cannot be textured but are much faster to render, as they are hardware supported gl points. By default these point particles are square, turn on hint-anti-alias to make them circular. @image{images/sprites}" " (define mynewshape (build-particles 100))\n")) ("build-locator" ("" "primitiveid-number" "A locator is an empty primitive, useful for parenting to (when you don't want to have the parent object visible). This primitive can only be visualised with (hint-origin) to display it's local transform origin." " (define mynewshape (build-locator))\n")) ("build-pixels" ("width-number height-number" "primitiveid-number" "Makes a new pixel primitive. A pixel primitive is used for making procedural textures, which can then be applied to other primitives. For this reason, pixel primitives probably wont be rendered much, but you can render them to preview the texture on a flat plane." " (define mynewshape (build-pixels 100 100))\n")) ("pixels-upload" ("" "void" "Uploads the texture data, you need to call this when you've finished writing to the pixelprim, and while it's grabbed." " (define mynewshape (build-pixels 100 100))\n (pixels-upload mynewshape)\n")) ("pixels->texture" ("pixelprimitiveid-number" "textureid-number" "Returns a texture you can use exactly like a normal loaded one." " (define mynewshape (build-pixels 100 100))\n (upload-pixels mynewshape)\n (texture (pixels->texture mynewshape))\n")) ("build-blobby" ("numinfluences subdivisionsvec boundingvec" "primitiveid-number" "Blobby primitives are a higher level implicit surface representation in fluxus which is defined using influences in 3 dimesional space. These infuences are then summed together, and a particular value is \"meshed\" (using the marching cubes algorithm) to form a smooth surface. The influences can be animated, and the smooth surface moves and deforms to adapt, giving the primitive it's blobby name. build-blobby returns a new blobby primitive. Numinfluences is the number of \"blobs\". Subdivisions allows you to control the resolution of the surface in each dimension, while boundingvec sets the bounding area of the primitive in local object space. The mesh will not be calculated outside of this area. Influence positions and colours need to be set using pdata-set." " (define mynewshape (build-blobby 7 (vector 30 30 30) (vector 3 3 3)))\n")) ("blobby->poly" ("blobbyprimitiveid-number" "polyprimid-number" "Converts the mesh of a blobby primitive into a triangle list polygon primitive." " (define mynewshape (blobby->poly myblobby))\n")) ("draw-instance" ("primitiveid-number" "void" "Copies a retained mode primitive and draws it in the current state as an immediate mode primitive." " (define mynewshape (build-cube))\n (colour (vector 1 0 0))\n (draw-instance mynewshape) ; draws a copy of mynewshape\n")) ("draw-cube" ("" "void" "Draws a cube in the current state in immediate mode primitive." " (define (render)\n     (draw-cube))\n (every-frame (render))\n")) ("draw-plane" ("" "void" "Draws a plane in the current state in immediate mode primitive." " (define (render)\n     (draw-plane))\n (every-frame (render))\n")) ("draw-sphere" ("" "void" "Draws a sphere in the current state in immediate mode primitive." " (define (render)\n     (draw-sphere))\n (every-frame (render))\n")) ("draw-cylinder" ("" "void" "Draws a cylinder in the current state in immediate mode primitive." " (define (render)\n     (draw-cylinder))\n (every-frame (render))\n")) ("destroy" ("primitiveid-number" "void" "Deletes a built primitive from the renderer. primitive." " (define mynewshape (build-sphere 10 10))\n (destroy mynewshape)\n")) ("poly-set-index" ("index-list" "void" "Switches the primitive to indexed mode, and uses the list as the index values for this primitive. primitive." " (clear)\n (define p (build-polygons 8 1))\n \n (grab p)\n ; setup the vertex data\n (pdata-set \"p\" 0 (vector -1 -1 -1))\n (pdata-set \"p\" 1 (vector  1 -1 -1))\n (pdata-set \"p\" 2 (vector  1 -1  1))\n (pdata-set \"p\" 3 (vector -1 -1  1))\n (pdata-set \"p\" 4 (vector -1  1 -1))\n (pdata-set \"p\" 5 (vector  1  1 -1))\n (pdata-set \"p\" 6 (vector  1  1  1))\n (pdata-set \"p\" 7 (vector -1  1  1))\n \n (hint-wire)\n (hint-unlit)\n \n ; connect the verts together into faces\n (poly-set-index (list 7 6 5 4  5 6 2 1 \n                        4 5 1 0  1 2 3 0\n                        3 7 4 0  6 7 3 2))\n \n (ungrab)\n")) ("poly-convert-to-indexed" ("" "void" "Converts the currently grabbed polygon primitive from raw vertex arrays to indexed arrays. This removes duplicate vertices from the polygon, making the pdata arrays shorter, which speeds up processing time." " (define mynewshape (build-sphere 10 10))\n (grab mynewshape)\n (poly-convert-to-indexed)\n (ungrab)\n")) ("build-copy" ("src-primitive-number" "primitiveid-number" "Returns a copy of a primitive" " (define mynewshape (build-sphere 10 10))\n (define myothernewshape (build-copy mynewshape))\n")) ("make-pfunc" ("name-string" "pfuncid-number" "Makes a new primitive function. pfuncs range from general purpose to complex and specialised operations which you can run on primitives. All pfuncs share the same interface for controlling and setting them up - pfunc-set! All pfunc types and arguments are as follows: arithmetic For applying general arithmetic to any pdata array operator string : one of add sub mul div src string : pdata array name other string : pdata array name (optional) constant float : constant value (optional) dst string : pdata array name genskinweights Generates skinweights - adds float pdata called \"s1\" -> \"sn\" where n is the number of nodes in the skeleton - 1 skeleton-root primid-number : the root of the bindpose skeleton for skinning sharpness float : a control of how sharp the creasing will be when skinned skinweights->vertcols A utility for visualising skinweights for debugging. no arguments skinning Skins a primitive - deforms it to follow a skeleton's movements. Primitives we want to run this on have to contain extra pdata - copies of the starting vert positions called \"pref\" and the same for normals, if normals are being skinned, called \"nref\". skeleton-root primid-number : the root primitive of the animating skeleton bindpose-root primid-number : the root primitive of the bindpose skeleton skin-normals number : whether to skin the normals as well as the positions" " (define mypfunc (make-pfunc 'arithmetic))\n")) ("pfunc-set!" ("pfuncid-number argument-list" "void" "Sets arguments on a primitive function. See docs for make-pfunc for all the arguments." " (define mypfunc (make-pfunc 'arithmetic))\n (pfunc-set! mypfunc (list 'operator \"add\"\n                           'src \"p\"\n                           'const 0.4\n                           'dst \"p\"))\n")) ("pfunc-run" ("id-number" "void" "Runs a primitive function on the currently grabbed primitive." " (define mypfunc (make-pfunc \"arithmetic\"))\n"))))) ("turtle" ("The turtle polybuilder is an experimental way of building polygonal objects using a logo style turtle in 3D space. As you drive the turtle around you can place vertices and build shapes procedurally. The turtle can also be used to deform existing polygonal primitives, by attaching it to objects you have already created." " (define (build n)\n     (turtle-reset)\n     (turtle-prim 4)\n     (build-loop n n)\n     (turtle-build))\n \n (define (build-loop n t)\n     (turtle-turn (vector 0 (/ 360 t) 0))\n     (turtle-move 1)\n     (turtle-vert)\n     (if (< n 1)\n         0\n         (build-loop (- n 1) t)))\n" (("turtle-prim" ("type-number" "void" "Starts building a new polygon primitive with the turtle. The type specifies the polygon face type and is one of the following: 0: triangle strip, 1: quad list, 2: triangle list, 3: triangle fan, 4: general polygon" " (turtle-prim 0)\n")) ("turtle-vert" ("" "void" "Creates a new vertex in the current position, or sets the current vertex if the turtle builder is attached." " (turtle-vert)\n")) ("turtle-build" ("" "primitiveid-number" "Builds the object with the vertex list defined and gives it to the renderer. Has no effect if the turtle builder is attached to a primitive." " (define mynewshape (turtle-build))\n")) ("turtle-move" ("distance-number" "void" "Moves the turtle forward in it's current orientation." " (turtle-move 1)\n")) ("turtle-push" ("" "void" "The turtle build has it's own transform stack. Push remembers the current position and orientation." " (turtle-push)\n")) ("turtle-pop" ("" "void" "The turtle build has it's own transform stack. Pop forgets the current position and orientation, and goes back to the state at the last push." " (turtle-pop)\n")) ("turtle-turn" ("rotation-vector" "void" "Rotates the turtle's orientation with the supplied euler angles (rotations in x, y and z)." " (turtle-turn (vector 45 0 0))\n")) ("turtle-reset" ("" "void" "Resets the current position and rotation of the turtle to the origin." " (turtle-reset)\n")) ("turtle-attach" ("primitiveid-number" "void" "Attaches the turtle to an existing poly primitive. This means you are able to deform an existing objects points using the turtle builder." " (define myshape (build-sphere 10 10))\n (turtle-attach myshape)\n")) ("turtle-skip" ("count-number" "void" "When attached, causes the turtle to skip vertices. This value may be negative, which will set the turtle to write to previous vertices." " (turtle-skip -1)\n")) ("turtle-position" ("" "count-number" "When attached, returns the current pdata index the turtle is writing to." " (display (turtle-position))(newline)\n")) ("turtle-seek" ("position-number" "void" "When attached, sets the absolute pdata index the turtle is writing to." " (turtle-seek 0)\n"))))) ("util-functions" ("Handy functions to make your life easier..." "" (("time" ("" "time-number" "Returns the number of seconds (+ fraction) since midnight January 1st 1970. This is the simpest animation source for your scripts." " (define (animate)\n     (rotate (vector (sin (time)) 0 0))\n     (draw-cube))\n (every-frame (animate))    \n")) ("delta" ("" "time-number" "Time in seconds since the last frame. Used to make animation frame rate independant." " (define (animate)\n     (rotate ((vector * (delta) 10) 0 0) \n     (draw-cube))\n (every-frame (animate))    \n")) ("flxrnd" ("" "random-number" "Returns a random number between 0 and 1." " (define (animate)\n     (colour (flxrnd) (flxrnd) (flxrnd)) \n     (draw-cube))\n (every-frame (animate))\n")) ("flxseed" ("seed-number" "void" "Seeds the random number generator so we can get the same sequence." " (define (animate)\n     (colour (flxrnd) (flxrnd) (flxrnd)) \n     (draw-cube))\n (flxseed 10) \n (every-frame (animate)) ; the same sequence of colours will be generated\n")) ("searchpaths" ("paths-list" "void" "Sets a list of search path strings to use for looking for fluxus related files, such as textures, shaders etc. Paths will be searched in order each time." " (searchpaths (list \"/path/to/my/textures\" \"/path/to/my/other/textures\"))\n")) ("fullpath" ("filename-string" "fullpath-string" "Searches the search paths for the specified file and returns the first location it finds." " (fullpath \"myfile\")\n")) ("framedump" ("filename" "void" "Saves out the current OpenGL front buffer to disk. Reads the filename extension to decide on the format used for saving, \"tif\", \"jpg\" or \"ppm\" are supported. This is the low level form of the frame dumping, use start-framedump and end-framedump instead." " (framedump \"picture.jpg\")\n"))))) ("osc" ("OSC stands for Open Sound Control, and is a widely used protocol for passing data between multimedia applications. Fluxus can send or receive messages." " An example of using osc to communicate between pd and fluxus.\n A fluxus script to move a cube based on incoming osc messages.\n -- osc.scm\n\n (define value 0)\n\n (define (test)\n     (push)\n     (if (osc-msg \"/zzz\")\n         (set! value (osc 0)))\n     (translate (vector 1 0 value))\n     (draw-cube)\n     (pop))\n \n (osc-source \"6543\")\n (every-frame (test))\n \n --- EOF\n A PD patch to send control messages to fluxus:\n --- zzz.pd\n #N canvas 618 417 286 266 10;\n #X obj 58 161 sendOSC;\n #X msg 73 135 connect localhost 6543;\n #X msg 58 82 send /zzz \\$1;\n #X floatatom 58 29 5 0 0 0 - - -;\n #X obj 58 54 / 100;\n #X obj 73 110 loadbang;\n #X connect 1 0 0 0;\n #X connect 2 0 0 0;\n #X connect 3 0 4 0;\n #X connect 4 0 2 0;\n #X connect 5 0 1 0;\n" (("osc-source" ("port-string" "void" "Starts up the osc server, or changes port. Known bug: seems to fail if you set it back to a port used previously." " (osc-source \"4444\")\t ; listen to port 4444 for osc messages\n")) ("osc-msg" ("name-string" "msgreceived-boolean" "Returns true if the message has been received since the last frame, and sets it as the current message for subsequent calls to (osc) for reading the arguments." " (cond \n     ((osc-msg \"/hello\")              ; if a the /hello message is recieved\n         (display (osc 1))(newline)))\t; print out the first argument\n")) ("osc" ("argument-number" "oscargument" "Returns the argument from the current osc message." " (cond \n     ((osc-msg \"/hello\")              ; if a the /hello message is recieved\n         (display (osc 1))(newline)))\t; print out the first argument\n")) ("osc-destination" ("port-string" "void" "Specifies the destination for outgoing osc messages. The port name needs to specify the whole url and should look something like this \"osc.udp://localhost:4444\"" " (osc-destination \"osc.udp:localhost:4444\")\n (osc-send \"/hello\" \"s\" (list \"boo!\"))  ; send a message to this destination\n")) ("osc-peek" ("" "msg-string" "This util function returns the name, and format string and number/string arguments of the last sent message as a string - for debugging your osc network." " (display (osc-peek))(newline) \n")) ("osc-send" ("name-string format-string argument-list" "void" "Sends an osc message with the argument list as the osc data. Only supports floats, ints and strings as data. The format-string should be composed of \"i\", \"f\" and \"s\", and must match the types given in the list. This could probably be removed by using the types directly, but doing it this way allows you to explicitly set the typing for the osc message." " (osc-destination \"osc.udp:localhost:4444\")\n (osc-send \"/hello\" \"sif\" (list \"boo!\" 3 42.3))  ; send a message to this destination\n"))))) ("scratchpad" ("The scratchpad is the fluxus editor and gl window." "" (("every-frame" ("callback-function" "void" "Sets a function to be called every time the render is about to draw a new frame." "   (define count 0)\n  \n   (define (myfunc)\n       (display count)(display \" frames have been rendered!\")\n       (newline)\n       (set! count (+ count 1)))\n  \n   (every-frame (myfunc)) \n  ")) ("clear" ("" "void" "Clears out the renderer of all objects and lights. Clears the physics system and resets the every-frame callback. Generally a Good Thing to put this at the beginning of scripts to make sure everything is cleared out each time you execute." "   (clear) ; without this we would accumulate a new cube every time F5 was pressed\n   (build-cube) \n  ")) ("start-framedump" ("name-string type-string" "void" "Starts saving frames to disk. Type can be one of \"tif\", \"jpg\" or \"ppm\". Filenames are built with the frame number added, padded to 5 zeros." "   (start-framedump \"frame\" \"jpg\") \n  ")) ("end-framedump" ("" "void" "Stops saving frames to disk." "   (end-framedump) \n  ")) ("set-help-locale!" ("locale-string" "void" "Sets the language for the documentation" "   (set-help-locale! \"pt\") \n  ")) ("help" ("function-string" "void" "Displays help information on a fluxus function. For running in the repl mainly." "   (help \"pop\") \n  ")) ("callback-override" ("callback-function" "void" "Allows you to override the frame callback, to control the rendering loop of fluxus in a more detailed way." "   (callback-override myfunc) \n  "))))) ("scratchpad-input" ("Input functions availible in the fluxus scratchpad." "" (("key-pressed" ("key-string" "boolean" "Returns true if the specified key is currently pressed down." " (if (key-pressed \"q\") (display \"q pressed!))\n")) ("keys-down" ("" "keys-list" "Returns a list of keys pressed down" " (display (keys-down))\n")) ("key-special-pressed" ("key-number" "boolean" "Returns true if the specified special key is currently pressed down. Special keys are ones which do not map to ascii values. The easiest way of finding what they are is to print out the result of key-special-pressed while holding down the key you are after." "\n (if (key-special-pressed 100) (display \"left cursor pressed\"))\n (if (key-special-pressed 102) (display \"right cursor pressed\"))\n (if (key-special-pressed 101) (display \"up cursor pressed\"))\n (if (key-special-pressed 103) (display \"down cursor pressed\"))\n")) ("keys-special-down" ("" "keys-list" "Returns a list of special keys pressed down" " (display (keys-special-down))\n")) ("mouse-x" ("" "coord-number" "Returns the x position of the mouse" " (display (mouse-x))\n")) ("mouse-y" ("" "coord-number" "Returns the y position of the mouse" " (display (mouse-y))\n")) ("mouse-button" ("" "boolean" "Returns true if the specifed mouse button is pressed" " (display (mouse-button 1))\n")) ("mouse-over" ("" "primitiveid-number" "Returns the object the mouse is currently over." " (grab (mouse-over))\n (colour (vector 1 0 0)) ; paints objects the mouse is over red\n (ungrab)\n"))))) ("scratchpad-docs" ("Some useful high level documentation lives here" "" (("tophelp" ("" "" "" " Fluxus documentation\n --------------------\n \"act of a flowing; a continuous moving on or passing by, as of a \n flowing stream; a continuous succession of changes\"\n\n Fluxus is a realtime rendering engine for livecoding in Scheme.\n You are encouraged to browse these texts while performing \n livecoding...\n\n The fluxus scratchpad has two modes of operation, the console\n (you are using this now) which allows you to enter commands and\n see the results immediately. The other mode is the editor which \n is more like a normal text editor - there are 9 workspaces,\n (which allow you to edit more than one script at once) switch to \n them using ctrl-1 to ctrl-9 and switch back to the console with \n ctrl-0.\n\n My apologies for the lack of copy/paste for the examples here,\n it's being worked on...\n\n More help topics:\n (help \"keys\") for keyboard commands for controlling fluxus \n (help \"console\") for more help on the console \n (help \"editor\") for more help on the livecoding editor\n (help \"camera\") for help on the camera controls \n (help \"language\") for more info on the fluxus commands\n (help \"misc\") for miscellaneous fluxus info \n (help \"toplap\") for the toplap manefesto\n (help \"authors\") who made this?\n")) ("keys" ("" "" "" " Fluxus keys\n -----------\n\n ctrl-f : Fullscreen mode.\n ctrl-w : Windowed mode.\n ctrl-h : Hide/show the text.\n ctrl-l : Load a new script (navigate with cursors and return).\n ctrl-s : Save current script.\n ctrl-d : Save as - current script (opens a filename dialog).\n ctrl-1 to 9 : Switch to selected workspace.\n ctrl-0 : Switch to the REPL.\n F3 : Resets the camera if you get lost.\n F5 : Executethe selected text, or all if none is selected.\n F9 : Randomise the text colour (aka the panic button)\n")) ("console" ("" "" "" " Fluxus console (or REPL)\n ------------------------\n\n If you press ctrl and 0, instead of getting another script \n workspace, you will be presented with a Read EvaluatePrint\n Loop interpreter, or repl for short. This is really just an \n interactive interpreter similar to the commandline, where \n you can enter scheme code for immediate evaluation. This code \n is evaluated in the same interpreter as the other scripts, so\n you can use the repl to debug or inspect global variables and \n functions they define. This window is also where error \n reporting is printed, along with the terminal window you \n started fluxus from.\n")) ("editor" ("" "" "" " Fluxus editor \n -------------\n\n When using the fluxus scratchpad, the idea is that you only \n need the one window to build scripts, or play live. f5 is the \n key that runs the script when you are ready.  Selecting some \n text (using shift) and pressing f5 will execute the selected \n text only. This is handy for reevaluating functions without \n running the whole script each time.\n\n Workspaces\n ----------\n\n The script editor allows you to edit 9 scripts simultaneously \n by using workspaces. To switch workspaces, use ctrl+number \n key. Only one can be run at once though, hitting f5 will \n execute the currently active workspace script. \n")) ("camera" ("" "" "" " Fluxus camera control\n ---------------------\n\n The camera is controlled by moving the mouse and pressing \n mouse buttons.\n\n Left mouse button: Rotate\n Middle mouse button: Move\n Right mouse button: Zoom\n")) ("misc" ("" "" "" " Fluxus init script\n ------------------\n Fluxus looks for a script in your home directory called\n .fluxus.scm which it will run if it is found. This is useful\n for putting init commands (like connecting to jack or setting\n the help text language etc)\n\n Frame rate throttling\n ---------------------\n By default fluxus throttles the framerate to around 40fps.\n to disable this (and run at 100% cpu), use desiredfps with\n some arbitrary large number:\n (desiredfps 100000000)\n To display the fps use (show-fps 1)\n\n Command line options\n ---------------------\n The easiest way to load a script into fluxus is to specify it on \n the command line, eg:\n $ fluxus myscript.scm\n Will launch fluxus and load the script into the editor.\n $ fluxus -x myscript.scm\n Will launch fluxus, load, hide and execute the script.\n\n Fluxus also contains a keypress and mouse event recorder for \n recording livecoding sessions:\n $ fluxus -r filename : record to keypresses file\n $ fluxus -p filename : playback from file\n $ fluxus -p filename -d time : seconds per frame time override for\n                                playback (for use with frame-dump)\n\n")) ("authors" ("" "" "" "\n \"Computers are useless. They can only give you answers\". \n     Pablo Picasso (1881 - 1973).\n\n Authors\n -------\n Alex Norman \n Artem Baguinski\n Dan Bethell\n Dave Griffiths \n Glauber Alex Dias Prado\n Nik Gaffney\n James Tittle\n\n")) ("language" ("" "" "" " Language Docs\n -------------\n \n Fluxus is comprised of a set of functions which\n extend Scheme for use in realtime computer graphics.\n \n Scheme itself is out of the scope of this documentation,\n but fluxus is a good way of learning it. I reccommend \n \"The Little Schemer\" by by Daniel P. Friedman and Matthias \n Felleisen.\n \n The functions are grouped into sections to make things\n a bit easier to find.\n\n (help \"sections\") for a list of all sections\n (help \"sectionname\") to find out more about a section\n (help \"functionname\") to find out more about a function\n\n The idea is that you can find a function you are interested \n in by doing something like this:\n\n (help \"sections\")\n    ... list of sections ...\n (help \"maths\")\n    ... description and list of maths functions ...\n (help \"vmul\")\n    ... details about the function with example ...\n\n\n")) ("toplap" ("" "" "" " TOPLAP MANEFESTO\n We demand:\n  * Give us access to the performer's mind, to the whole human \n    instrument.\n  * Obscurantism is dangerous. Show us your screens.\n  * Programs are instruments that can change themselves\n  * The program is to be transcended - Artificial language is the way.\n  * Code should be seen as well as heard, underlying algorithms viewed as well as their visual outcome.\n  * Live coding is not about tools. Algorithms are thoughts. \n    Chainsaws are tools. That's why algorithms are sometimes \n    harder to notice than chainsaws.\n \n We recognise continuums of interaction and profundity, but prefer: \n  * Insight into algorithms\n  * The skillful extemporisation of algorithm as an \n    expressive/impressive display of mental dexterity\n  * No backup (minidisc, DVD, safety net computer)\n \n We acknowledge that:\n  * It is not necessary for a lay audience to understand the \n    code to appreciate it, much as it is not necessary to know \n    how to play guitar in order to appreciate watching a guitar \n    performance.\n  * Live coding may be accompanied by an impressive display of \n    manual dexterity and the glorification of the typing interface.\n  * Performance involves continuums of interaction, covering \n    perhaps the scope of controls with respect to the parameter \n    space of the artwork, or gestural content, particularly \n    directness of expressive detail. Whilst the traditional \n    haptic rate timing deviations of expressivity in \n    instrumental music are not approximated in code, why repeat \n    the past? No doubt the writing of code and expression of \n    thought will develop its own nuances and customs.\n \n Performances and events closely meeting these manifesto \n conditions may apply for TOPLAP approval and seal. \n"))))) ("scratchpad-camera" ("Camera functions availible in the fluxus scratchpad." "" (("reset-camera" ("" "void" "Resets the camera transform, useful if it becomes trashed, or you get lost somewhere in space. Also turns off camera locking to objects with (lock-camera)" " ; ruin the camera transform\n (set-camera-transform (vector 123 41832 28 0.2 128 0.001 123 41832 28 0.2 128 0.001 0.2 100 13 1931)) \n ; set it back to the starting position/orientation\n (reset-camera) \n")) ("set-camera-transform" ("transform-matrix" "void" "Overrides and locks the camera transform with your own. To unlock again call reset-camera" " (set-camera-transform (mtranslate 0 0 -10)) \n")) ("get-camera-transform" ("" "transform-matrix" "Returns the current camera transform. To unlock again call reset-camera" " (define tx (get-camera-transform))\n"))))))) ("pt" (("audio" ("Esta parte do fluxus é responsável por capturar o som entrando, e processar ele em dados harmonicos, usando fft (Fast Fourier Transform). As harmonicas são bandas de frequência em que o som é dividido, dando alguma indicação da qualidade do som. É o mesmo que você ve num equalisador gráfico - de fato, um dos scripts de exemplo (bars.scm) age como tal, e pode ser usado para testar se o áudio está funcionando." " (start-audio \"alsa_pcm:capture_1\" 1024 44100)\n (define (animate)\n\t\t(colour (vector (gh 1) (gh 2) (gh 3))) ; make a colour from the harmonics, and set it to be the current colour \n\t\t(draw-cube)) ; draw a cube with this colour\n (every-frame (animate))\n" (("start-audio" ("string-porta-do-jack número-tamanho-buffer número-taxa-amostragem" "void" "Inicia o áudio com as configurações específicadas, você precisa chamar isto primeiro, ou colocar isto em $HOME/.fluxus.scm para chamar automaticamente na inícialização. Tenha a porta do jack como uma string vazia (\"\") e ele não vai tentar conectar em nada para você. Que pode então usar qjackctrl ou equivalente para fazer a conexão manualmente. Fluxus lè uma única fonte mono." " (start-audio \"alsa_pcm:capture_1\" 1024 44100)\n EndSectionDoc\nScheme_Object *start_audio(int argc, Scheme_Object **argv)\n{\n\tMZ_GC_DECL_REG(1);\n\tMZ_GC_VAR_IN_REG(0,argv); \n\tMZ_GC_REG();\n\t\n\tif (!SCHEME_CHAR_STRINGP(argv[0]))scheme_wrong_type(\"start-audio\", \"string\", 0, argc, argv);\n\tif (!SCHEME_NUMBERP(argv[1]))scheme_wrong_type(\"start-audio\", \"number\", 1, argc, argv);\n\tif (!SCHEME_NUMBERP(argv[2]))scheme_wrong_type(\"start-audio\", \"number\", 2, argc, argv);\n\t\n\tif (Audio==NULL)\n\t{\n\t\tchar *name = scheme_utf8_encode_to_buffer(SCHEME_CHAR_STR_VAL(argv[0]),SCHEME_CHAR_STRLEN_VAL(argv[0]),NULL,0);\t\tAudio = new AudioCollector(name,(unsigned int)scheme_real_to_double(argv[1]),(int)scheme_real_to_double(argv[2]));\n\t}\n\t\n\tMZ_GC_UNREG();\n\treturn scheme_void;\n}\n\n StartFunctionDoc-en\n gh harmonic-number\n Returns: harmonic-real\n Description:\n Fluxus converts incoming audio into harmonic frequencies, which can then be plugged into your \n animations using this command. There are 16 harmonic bands availible, the harmonic-value argument \n will be wrapped around if greater or less than 16, so you can use this command without worrying \n about out of range errors.\n Example: (define (animate)\n\t\t(colour (vector (gh 1) (gh 2) (gh 3))) ; make a colour from the harmonics, and set it to be the current colour \n\t\t(draw-cube)) ; draw a cube with this colour\n (every-frame (animate))\n")) ("gh" ("número-harmonico" "void" "Fluxus converte áudio entrando em frequências harmônicas, que pode então ser ligada em suas animações através desse comando. Existem 16 bandas harmônicas disponiveis, o argumento do valor-harmônico vai ser arredondado se maior ou menor que 16, então você pode usar esse comando sem se preocupar sobre erros fora do alcance." " (define (animate)\n\t\t(colour (vector (gh 1) (gh 2) (gh 3))) ; make a colour from the harmonics, and set it to be the current colour \n\t\t(draw-cube)) ; draw a cube with this colour\n (every-frame (animate))\n EndSectionDoc\nScheme_Object *get_harmonic(int argc, Scheme_Object **argv)\n{\n\tMZ_GC_DECL_REG(1);\n\tMZ_GC_VAR_IN_REG(0,argv); \n\tMZ_GC_REG();\n\tif (!SCHEME_NUMBERP(argv[0]))scheme_wrong_type(\"gh\", \"number\", 0, argc, argv);\n\tif (Audio!=NULL)\n\t{\t\n\t\tMZ_GC_UNREG();\n    \treturn scheme_make_double(Audio->GetHarmonic((int)scheme_real_to_double(argv[0])));\n\t}\n\tMZ_GC_UNREG();\n\treturn scheme_make_double(0);\n}\n\n StartFunctionDoc-en\n gain gain-number\n Returns: void\n Description:\n Sets the gain level for the fft sound, it's 1 by default.\n Example: (gain 100) ; too quiet?!\n")) ("gain" ("número-gain" "void" "Ajusta o nível de amplificação para o som fft, é 1 por padrão." " (gain 100) ; muito quieto?!\n EndSectionDoc\nScheme_Object *gain(int argc, Scheme_Object **argv)\n{\n\tMZ_GC_DECL_REG(1);\n\tMZ_GC_VAR_IN_REG(0,argv); \n\tMZ_GC_REG();\n\tif (!SCHEME_NUMBERP(argv[0]))scheme_wrong_type(\"gain\", \"number\", 0, argc, argv);\n\tif (Audio!=NULL)\n\t{\t\n\t\tAudio->SetGain(scheme_real_to_double(argv[0]));\n\t}\n\tMZ_GC_UNREG();\n    return scheme_void;\n}\n\n StartFunctionDoc-en\n process wavfile-string\n Returns: void\n Description:\n This command temporarally disables the realtime reading of the input audio stream and reads a \n wav file instead. For use with the framedump command to process audio offline to make music \n videos. The advantage of this is that it locks the framerate so the right amount of audio gets\n read for each frame - making syncing of the frames and audio files possible.\n Example: (process \"somemusic.wav\") ; read a precorded audio file\n")) ("process" ("wavfile-string" "void" "Este comando desativa temporariamente a leitura em tempo real da entrada da pista de áudio e lê um arquivo wav ao invés. Para usar com o comando framedump para processar audio offline para fazer videos musicais. A vantagem disto é que ele trava a taxa de quadros então a quantidade certa de áudio é lida para cada quadro - fazendo com que a sincrônia entre quadros e audio seja possível." " (process \"somemusic.wav\") ; read a precorded audio file\n EndSectionDoc\nScheme_Object *process(int argc, Scheme_Object **argv)\n{\n\tMZ_GC_DECL_REG(1);\n\tMZ_GC_VAR_IN_REG(0,argv); \n\tMZ_GC_REG();\n\tif (!SCHEME_CHAR_STRINGP(argv[0]))scheme_wrong_type(\"process\", \"string\", 0, argc, argv);\n\tchar *wavname=scheme_utf8_encode_to_buffer(SCHEME_CHAR_STR_VAL(argv[0]),SCHEME_CHAR_STRLEN_VAL(argv[0]),NULL,0);\tif (Audio!=NULL)\n\t{\t\n\t\tAudio->Process(wavname);\n\t}\n\tMZ_GC_UNREG();\n    return scheme_void;\n}\n\n StartFunctionDoc-en\n smoothing-bias value-number\n Returns: void\n Description:\n A kind of weighted average for the harmonic bands which smooth them out over time. \n This setting defaults to 1.5. The best value really depends on the quality of the music, \n and the buffer sizes, and ranges from 0 -> 2. It's more obvious if you give it a try \n with the bars.scm script\n Example: (smoothing-bias 0) ; no smoothing\n")) ("smoothing-bias" ("número-valor" "void" "Uma espécie de média balanceada para as bandas harmônicas que as acalmam com o tempo. Esta opção é por padrão definida como 1.5. O melhor valor realmente depende da qualidade da música, e do tamanho do buffer, e varia de 0 -> 2. Fica mais óbvio se você tentar com o script bars.scm" " (smoothing-bias 0) ; no smoothing\n EndSectionDoc\nScheme_Object *smoothing_bias(int argc, Scheme_Object **argv)\n{\n\tMZ_GC_DECL_REG(1);\n\tMZ_GC_VAR_IN_REG(0,argv); \n\tMZ_GC_REG();\n\tif (!SCHEME_NUMBERP(argv[0]))scheme_wrong_type(\"smoothing-bias\", \"number\", 0, argc, argv);\n\tif (Audio!=NULL)\n\t{\t\n\t\tAudio->SetSmoothingBias(scheme_real_to_double(argv[0]));\n\t}\n\tMZ_GC_UNREG();\n    return scheme_void;\n}\n\n StartFunctionDoc-en\n update-audio\n Returns: void\n Description:\n Updates the audio subsytem. This function is called for you (per frame) in fluxus-canvas.ss.\n Example: (smoothing-bias 0) ; no smoothing\n")) ("update-audio" ("" "void" "Atualiza o subsistema de áudio. Esta função é chamada para você (por frame) no fluxus-canvas.ss" "\n EndSectionDoc\nScheme_Object *update_audio(int argc, Scheme_Object **argv)\n{\n\tif (Audio!=NULL)\n\t{\t\n\t\tAudio->GetFFT();\n\t}\n    return scheme_void;\n}\n\n/\n\nScheme_Object *scheme_reload(Scheme_Env*env)\n{\n\tScheme_Env*menv=NULL;\n\tMZ_GC_DECL_REG(2);\tMZ_GC_VAR_IN_REG(0,env);\n\tMZ_GC_VAR_IN_REG(1,menv);\n\tMZ_GC_REG();\t add all the modules from this extension\n\tmenv=scheme_primitive_module(scheme_intern_symbol(\"fluxus-audio\"), env);\n\n\tscheme_add_global(\"start-audio\", scheme_make_prim_w_arity(start_audio, \"start-audio\", 3, 3), menv);\n\tscheme_add_global(\"gh\", scheme_make_prim_w_arity(get_harmonic, \"gh\", 1, 1), menv);\n\tscheme_add_global(\"gain\", scheme_make_prim_w_arity(gain, \"gain\", 1, 1), menv);\n\tscheme_add_global(\"process\", scheme_make_prim_w_arity(process, \"process\", 1, 1), menv);\n\tscheme_add_global(\"smoothing-bias\", scheme_make_prim_w_arity(smoothing_bias, \"smoothing-bias\", 1, 1), menv);\n\tscheme_add_global(\"update-audio\", scheme_make_prim_w_arity(update_audio, \"update-audio\", 0, 0), menv);\n\n\tscheme_finish_primitive_module(menv);\t\n \tMZ_GC_UNREG();\n\t\n\treturn scheme_void;\n}\n\nScheme_Object *scheme_initialize(Scheme_Env*env)\n{\n\treturn scheme_reload(env);\n}\n\nScheme_Object *scheme_module_name()\n{\n\treturn scheme_intern_symbol(\"fluxus-audio\");\n"))))) ("renderer" ("Estes comandos são os controladores de baixo nível do renderizador. Você não deveria usá-los a menos que esteja implementando um renderizador do fluxus fora da interface scratchpad." "" (("make-renderer" ("" "número-render-id" "Faz um novo renderizador de gráfico da cena [scenegraph]" " (make-renderer) \n")) ("renderer-grab" ("número-id-renderizador" "void" "Faz este renderizador o atual contexto para comandos." " (renderer-grab renderer) \n")) ("renderer-ungrab" ("" "void" "Pop o contexto da pilha do renderizador." " (renderer-grab renderer) \n")) ("fluxus-render" ("" "void" "Limpa o buffer de fundo, e renderiza tudo." " (fluxus-render)\n")) ("tick-physics" ("" "void" "Atualiza o sistema de física." " (tick-physics) \n")) ("render-physics" ("" "void" "Renderiza o sistema de física (para gráficos ajudantes)." " (render-physics)\n")) ("reset-renderers" ("" "void" "Deleta todos os renderizadores e faz um novo como padrão." " (reset-renderers)\n")) ("reshape" ("número-largura número-altura" "void" "Chama reshapa no render atual." " (reshape 100 100) \n")) ("fluxus-init" ("" "void" "Inicia todo o sistema de renderização, só precisa ser chamado uma vez." " (fluxus-init)\n"))))) ("global-state" ("Estado global é realmente qualquer coisa que controla o renderizador globalmente, então ele afeta todas as primitivas ou controla o renderizador diretamente - p.e. controle de câmera ou efeitos de tela cheia como \"embaçamento\"." "" (("clear-engine" ("" "void" "Limpa o renderizador, e o sistema de física. Este comando não deve ser chamado diretamente, use clear ao invés, já que limpa algumas outras coisas também, e chama clear-engine ele mesmo." " (clear-engine) ; woo hoo!\n")) ("blur" ("número-quantidade" "void" "Ajusta a opção de blur na tela inteira. Menos é mais, mas se você ajustar isto muito baixo vai fazer com que a edição na tela fique impossível de ler, então salve seus scripts primeiro :)." " (blur 0.1) ; para belos rastros\n")) ("fog" ("cor-nuvem-vetor número-quantidade número-ínicio númeor-final" "void" "Ajusta os paramêtros da neblina pra dar uma indicação de profundidade visual (perspectiva aérea no jargão de pintores). Isto pode obscurecer a edição na tela, então mantenha a quantidade baixa." " (clear-colour (vector 0 0 1)) ; fica legal se o fundo de tela bate.\n (fog (vector 0 0 1) 0.01 1 100) ; neblina azul\n")) ("show-axis" ("número-mostrar" "void" "Mostra os eixos de origem do espaço usado;" " (show-axis 1)\n")) ("show-fps" ("número-mostrar" "void" "Mostra uma contaem de fps na parte inferior esquerda da tela." " (show-fps 1)\n")) ("lock-camera" ("número-id-primitiva" "void" "Trava a transformação da camera em cima da transformação do objeto específicado. É como parentear a câmera ao objeto. Esta é a forma mais fácil de dirigir a câmera proceduralmente. Use um número id de 0 para destravar a câmera." " (clear)\n (define obj (build-cube)) ; make a cube for the camera to lock to\n \n (push) ; make a background cube so we can tell what's happening\n (hint-wire)  \n (hint-unlit) \n (colour (vector 0 0.4 0))\n (scale (vector -50 -50 -50))\n (build-cube)\n (pop)\n \n (lock-camera obj) ; lock the camera to our first cube\n \n (define (animate)\n     (grab obj)\n     (rotate (vector 1 0 0)) ; rotate the cube\n     (ungrab))\n \n (every-frame (animate))\n")) ("camera-lag" ("número-quantidade" "void" "O travamento da câmera tem um atraso construído junto o que significa que o movimento vai ser macio em relativo a primitiva ao qual ela está travada." " (clear)\n (define obj (build-cube)) ; make a cube for the camera to lock to\n \n (push) ; make a background cube so we can tell what's happening\n (hint-wire)\n (hint-unlit)\n (colour (vector 0 0.4 0))\n (scale (vector -50 -50 -50))\n (build-cube)\n (pop)\n \n (lock-camera obj) ; lock the camera to our first cube\n (camera-lag 0.1)  ; set the lag amount, this will smooth out the cube jittery movement\n \n (define (animate)\n     (grab obj)\n     (identity)\n     (translate (vector (modulo (round (inexact->exact (time))) 6) 0 0)) ; make a jittery movement\n     (ungrab))\n \n (every-frame (animate))\n")) ("load-texture" ("pngnomedoarquivo-string" "void" "Carrega uma imagem do disco, converte esta a uma textura e retorna o número id. O carregamento da textura se dá no cache de memória, então repetidamente chamar esta função não vai causar que carregue de novo. Use force-load-texture se você está mudando a textura enquanto o script estiver rodando. O png pode ser RGB ou RGBA para usar transparência alpha." " (texture (load-texture \"mytexture.png\"))\n (build-cube) ; o cubo vai ser mapeado com a textura da imagem\n")) ("force-load-texture" ("pngnomedoarquivo-string" "void" "Carregamento de texturas do disco sem cache, converte esta a uma textura e retorna o número id. Útil se você está mudando a textura enquanto está rodando o script, ou então usando load-texture, o que vai ser muito mais rápido. A png pode ser RGB ou RGBA para usar transparência alpha." " (texture (force-load-texture \"mytexture.png\"))\n (build-cube) ; o cubo vai ser mapeado com a textura da imagem\n")) ("frustum" ("número-topo número-baixo número-esquerda número-direita" "void" "Ajusta o frustum da camera, e portanto o quociente de aspecto do frame." " (frustum -1 1 -0.75 0.75) ; definições padrão\n")) ("clip" ("número-frente número-trás" "void" "Ajusta os planos de clipagem da frente e de trás para o frustum da câmera, portanto o ângulo de visão. Mude a distância da frente do clip para alterar a perspectiva de telephoto para fisheye." " (clip 1 10000) ; default settings\n")) ("ortho" ("" "void" "Ajusta a projeção ortográfica - p.e. sem perspectiva." " (ortho)\n")) ("persp" ("" "void" "Ajusta a projeção como perspectiva (o padrão) depois que ortho foi acionada." " (persp)\n")) ("set-ortho-zoom" ("número-quantidade" "void" "Ajusta o nível de zoom para a projeção ortográfica." " (set-ortho-zoom 2)\n")) ("backfacecull" ("número-ajuste" "void" "Liga ou desliga o corte de face-traseira. Backface culling acelera a renderização removendo faces não orientadas em direção da câmera. É ligado por padrão, mas isto não é desejado sempre, eg para poligonos com dupla face." " (backfacecull 0)\n")) ("clear-colour" ("vetor-cor" "void" "Ajusta a cor que vai limpar o renderizador, isto forma a cor do fundo da cena." " (clear-colour (vector 1 0 0)) ; RED!!!\n")) ("clear-frame" ("número-ajuste" "void" "ajusta a limpeza do frame, desligado ou ligado." " (clear-frame 0)\n")) ("clear-zbuffer" ("número-ajuste" "void" "Ajusta a limpeza do zbuffer, desligado ou ligado." " (clear-zbuffer 0)\n")) ("clear-accum" ("número-ajuste" "void" "Ajusta a limpeza do buffer de acumulação, ligado ou desligado" " (clear-accum 1)\n")) ("get-camera" ("" "vetor-matriz" "Pega a transformação da camera. Esta é a função de baixo nível, use get-camera-transform ao invés." " (get-camera)\n")) ("get-locked-matrix" ("" "vetor-matriz" "Pega a matriz de tranformação da câmera travada. Leva em consideração o atraso." " (get-locked-matrix)\n")) ("set-camera" ("" "void" "Ajusta a matriz de transformação da câmera. Esta é a interface de baixo nível usada por set-camera-transform, a qual você devia usar geralmente ao invés." " (set-camera)\n")) ("get-projection-transform" ("" "matriz-de-projeção" "Pega a matriz de projeção atual." " (get-projection-transform)\n")) ("get-screen-size" ("" "vetor-tamanho" "Retorna um vetor contendo a atual largura e altura da janela" " (get-screen-size)\n")) ("set-screen-size" ("vetor-tamanho" "void" "Ajusta a altura e largura da janela." " (set-screen-size (vector 10 10)) ; small window time :)\n")) ("select" ("número-janelaposX número-janelaposY número-tamanho-pixel" "número-id-primitiva" "Olha na região específicada e retorna a id da primitiva mais próxima à renderização da câmera lá, ou 0 se não existente." " (display (select 10 10 2))(newline)\n")) ("desiredfps" ("número-fps" "void" "Desacelera o renderizador de forma a não pegar 100% de cpu. Isto dá um limite acima na taxa de fps, o que não completamente bate o número dado, mas nós estamos trabalhando nisto..." " (desiredfps 100000) ; faz fluxus renderizar tão rápido quanto pode\n                     ; e levar 100% de cpu.\n")) ("draw-buffer" ("nome-buffer" "void" "Seleciona qual buffer para desenhar, se em modo estéreo você iria fazer 'back-right e 'back-left" " (draw-buffer 'back)\n")) ("read-buffer" ("nome-buffer" "void" "Seleciona qual buffer para ler." " (read-buffer 'back)\n")) ("set-stereo-mode" ("modo" "bool" "seleciona qual modo estéreo a usar, atualmente somente 'cristal-eyes e 'no-stereo são suportados o retorno indica se a operação foi bem sucedida ou não 'crystal-eyes vai retornar falso se você não tem uma janela estéreo." " (set-stereo-mode 'crystal-eyes)\n")) ("set-colour-mask" ("vetor" "void" "Ajusta a máscara de cor dando a esta um quatérnio de booleanos que correspondem aos canais vermelho, verde, azul e alpha respectivamente depois desta operação você vai ver apenas aquelas cores que você ajustar como verdadeiras (isto é útil apenas para estéreo com óculos azul-vermelhos)" " (set-colour-mask #(#t #f #f #t))\n")) ("shadow-light" ("número-ajuste" "void" "Ajusta a luz para usar na geração de sombras, ajuste para 0 para desativar renderização de sombras." " (shadow-light 1)\n")) ("shadow-length" ("número-ajuste" "void" "Ajusta o alcance do volume da renderização da sombra." " (shadow-length 10)\n")) ("shadow-debug" ("número-ajuste" "void" "Liga debug na renderização do volume da sombra." " (shadow-debug 1)\n")) ("accum" ("simbolo-modo número-valor" "void" "Controla o buffer de acumulação (somente chama glAccum embaixo do tapete). Símbolos possíveis são: accum load return add mult" " (accum 'add 1)\n"))))) ("lights" ("Sem luzes você não seria capaz de ver nada. Por sorte fluxus te dá uma de graça por padrão, uma luz pontual difusa e branca fixa à camêra. Para iluminações mais interessantes, você vai precisar destas funções. Usando as funções fixas de pipeline gráfico padrão, falando simplisticamente, OpenGL multiplica estes valores pela superficie do material (ajustado com comandos do estado local [local state] como ambient e diffuse) e o valor cor da textura dando a cor final." "" (("make-light" ("simbolo-tipo simbolo-travado-camera[cameralocked]" "lightid-number" "Faz uma luz nova. O tipo pode ser um destes: point, directional ou spot. Se a string de cameralocked não estiver livre então esta vai ser fixa à câmera, e mexer ao redor quando você mover a câmera." " (make-light 'spot 'locked)\n")) ("light-ambient" ("número-id-luz colour" "void" "Ajusta a contribuição do ambiente para a luz especificada." " (light-ambient mylight (vector 1 1 1)) ; a boring light\n")) ("light-diffuse" ("número-id-luz colour" "void" "Ajusta a contribuição difusa para a luz especificada." " (light-diffuse mylight (vector 1 1 1)) \n")) ("light-specular" ("número-id-luz colour" "void" "Ajusta a contribuição especular para a luz específicada." " (light-specular mylight (vector 1 1 1)) \n")) ("light-position" ("número-id-luz vetor-posição" "void" "Ajusta a posição da luz especificada. Em espaço global se livre, em espaço da câmera se fixa." " (light-position mylight (vector 0 100 0)) \n")) ("light-spot-angle" ("número-id-luz número-ângulo" "void" "Ajusta o ângulo do cone da luz spot especificada. Se não é uma luz spot este comando não tem nenhum efeito." " (light-position mylight (vector 0 100 0)) \n")) ("light-spot-exponent" ("número-id-luz número-exponente" "void" "Ajusta a exponencial da luz spot (dispersão do cone) específicada. Não funciona com outra lâmpada senão a spot." " (light-spot-exponent mylight 0.1) \n")) ("light-attenuation" ("número-id-luz símbolo-tipo número-atenuação" "void" "Ajusta a atenuação da luz (decaimento com distância) da luz especificada. O símbolo-tipo, pode ser um dos: constant, linear ou quadratic." " (light-attenuation 'constant 0.1) \n")) ("light-direction" ("número-id-luz vetor-direção" "void" "Ajusta a direção da luz direcional. Se não é uma luz direcional, este comando não tem efeito." " (light-spot-exponent mylight 0.1) \n"))))) ("local-state" ("As funções de estado local controlam a renderização ou do estado corrente - ou do estado da primitiva correntemente pega(grabbed). Em Fluxus estado significa significa a forma que as coisas são mostradas, tanto ligando/desligando opções de render, mudando o estilo de diferentes opções, ou alterando a transformação corrente." "" (("push" ("" "void" "Empurra uma cópia do estado corrente de desenho para o topo da pilha. O estado de desenho contém informação sobre coisas como cor corrente, transformação e dicas(hints)." " (colour (vector 1 0 0)) ; aplica cor corrente pra vermelho\n (push)                  ; copia e empurra estado de desenho\n (colour (vector 0 1 0)) ; aplica cor corrente pra verde\n (draw-cube)\t\t\t   ; desenha um cubo verde\n (pop)\t\t\t\t   ; esquece estado de desenho antigo\n ; cor corrente é vermelha de novo.\n")) ("pop" ("" "void" "Destrói o estado de desenho corrente, e aplica para o estado corrente o antigo anteriormente empurrado na pilha. O estado de desenho contém informação sobre coisas como cor corrente, transformação e dicas(hints)." " (colour (vector 1 0 0)) ; aplica cor corrente pra vermelho\n (push)                  ; copia e empurra estado de desenho\n (colour (vector 0 1 0)) ; aplica cor corrente pra verde\n (draw-cube)             ; desenha um cubo verde\n (pop)                   ; esquece estado de desenho antigo\n ; cor corrente é vermelha de novo.\n")) ("grab" ("id-do-objeto" "void" "Pega o objeto especificado. Uma vez que o objeto foi pego seu estado pode ser modificado usando os mesmos comandos usados pra ajustar o estado de desenho atual. (ungrab) precisa ser usado para retornar ao estado de desenho normal. \"grabbing\" também pode ser \"pilhado\", neste caso (ungrab) pula para a próxima primitiva pega." " (colour (vector 1 0 0))      ; aplica a cor atual para vermelho\n (define mycube (build-cube)) ; faz um cubo vermelho\n (grab mycube)  \t\t\t\t       \n (colour (vector 0 1 0)) ; aplica a cor do cubo como verde \n (ungrab)\t\t\t\t   ; retorna ao estado normal\n")) ("ungrab" ("" "void" "Solta o objeto pego atualmente, e ou retorna ao estado de desenho normal, ou tira a ultima primitiva pega do topo da pilha." " (colour (vector 1 0 0))      ; aplica a cor atual para vermelho\n (define mycube (build-cube)) ; faz um cubo vermelho\n (grab mycube)  \t\t\t\t       \n (colour (vector 0 1 0)) ; aplica a cor verde ao cubo \n (ungrab)\t\t\t\t   ; retorna ao estado normal\n")) ("apply" ("id-do-objeto" "void" "Aplica a transformação corrente às posições dos vértices do objeto dado e ajusta sua tranformação para identidade." " (rotate (vector 45 0 0))     \n (define mycube (build-cube)) ; faz um cubo com uma rotação\n (apply mycube)  \t      ; aplica a rotação aos pontos do cubo\n")) ("opacity" ("valor" "void" "Ajusta a opacidade do estado de desenho atual, ou da primitiva pega atualmente." " (opacity 0.5)     \n (define mycube (build-cube)) ; faz um cubo semi-transparente.\n")) ("shinyness" ("valor" "void" "Ajusta o brilho do estado atual de desenho, ou da primitiva atualmente pega. Esse valor ajusta a densidade do brilho especular." " (shinyness 100)     \n (specular (vector 1 1 1)) ; ajusta a cor especular\n (define mysphere (build-sphere 10 10)) ; makes a shiny cube \n")) ("colour" ("vetor-cor" "void" "Ajusta a cor do estado de desenho atual, ou a primitiva atualmente pega." " (colour (vector 1 0.5 0.1)) ; mmm laranja...   \n (define mycube (build-cube)) ; faz um cubo laranja \n")) ("wire-colour" ("vetor-cor" "void" "Ajusta a cor do \"wire frame\" do estado de desenho atual, ou a primitiva atualmente pega. Visível com (hint-wire) na maioria das primitivas." " (wire-colour (vector 1 1 0)) ; ajusta a cor do fio como amarelo\n (hint-wire)   \n (define mycube (build-cube)) ; faz um cubo com wireframe amarelo\n")) ("specular" ("vetor-cor" "void" "Ajusta a cor especular do estado de desenho corrente, ou o objeto atualmente pego." " (specular (vector 0 0 1)) ; ajusta azul como a cor especular\n (define mysphere (build-sphere 10 10)) ; faz uma esfera azul brilhante.\n")) ("ambient" ("vetor-cor" "void" "Ajusta a cor ambiente do estado de desenho corrente, ou a primitiva atualmente pega." " (ambient (vector 0 0 1)) ; ajusta a cor ambiente como azul\n (define mysphere (build-sphere 10 10)) ; faz uma chata esfera azul \n")) ("opacity" ("valor" "void" "Ajusta a cor emissiva do estado de desenho atual, ou da primitiva atualmente pega." " (emissive (vector 0 0 1)) ; ajusta a cor emissiva para azul\n (define mysphere (build-sphere 10 10)) ; faz uma esfera azul brilhante \n")) ("identity" ("" "void" "Ajusta a transformação do estado de desenho para identidade, no estado de pilha, ou a primitiva atualmente pega." " (push)\n (scale (vector 2 2 2)) ; ajusta o tamanho atual pro dobro em cada dimensão\n (define mycube (build-cube)) ; faz um cubo aumentado\n (pop)\n (grab mycube)\n (identity) ; apaga a transformação e coloca o cubo de volta ao seu\n            ; estado original\n (ungrab)\n")) ("concat" ("matrix" "void" "Concatena (multiplica) uma matriz para o estado de desenho atual ou primitiva pega." " (define mymatrix (mrotate (vector 0 45 0))) ; faz uma matriz\n (concat mymatrix) ; concat ela no estado atual\n (build-cube) ; faz um cubo com esta rotação\n")) ("translate" ("vetor" "void" "Aplica uma translação ao estado de desenho atual ou primitiva pega" " (transform (vector 0 1.4 0)) ; translada a transformação atual pra\n                              ; cima um pouco\n (build-cube) ; constrói um cubo com esta transformação\n")) ("rotate" ("vetor-ou-quaternion" "void" "Aplica uma rotação ao estado de desenho atual ou primitiva pega." " (rotate (vector 0 45 0)) ; vira 45 graus no eixo Y \n (build-cube) ; constrói um cubo com esta transformação\n")) ("scale" ("vetor" "void" "Aplica uma escalagem ao estado de desenho atual ou primitiva pega." " (scale (vector 0.5 0.5 0.5)) ; escala a tranformação atual para\n                              ; metade do tamanho\n (build-cube) ; constrói um cubo com esta transformação\n")) ("get-transform" ("" "vetor-matriz" "Retorna uma matriz representando o estado de tranformação corrente ou para a primitiva pega." " (translate (vector 1 0 0))\n (display (get-transform))(newline) ; imprime a transformação corrente\n (define shape (build-sphere 10 10))\n (grab shape)\n (translate (vector 0 1 0))\n (display (get-transform))(newline) ; imprime a transformação da forma\n (ungrab)\n")) ("parent" ("id-primitiva" "void" "Parenteia a primitiva correntemente pega à primitiva pai dada. A primitiva corrente vai ser agora movida junto com o pai por adquirir todas as suas transformações." " (define parent-prim (build-cube)) ; make a parent cube\n (translate (vector 2 0 0)) ; move a bit in x\n (parent parent-prim) ; set parent-prim as the current parent\n (define child-prim (build-cube)) ; make a child cube\n (grab parent-prim) \n (rotate (vector 0 45 0)) ; the child will now be moved by this transform in addition to its own\n (ungrab)\n")) ("line-width" ("valor" "void" "Ajusta a largura da linha (em espaço de tela) do estado de desenho corrente, ou da primitiva correntemente pega, Afeta wireframe e afins." " (line-width 5)\n (hint-wire)\n (build-sphere 10 10) ; faz uma esfera com um denso wireframe\n")) ("point-width" ("value" "void" "Ajusta o tamanho do ponto (em espaço na tela) do estado de desenho corrente, ou da primitiva pega. Afeta a renderização de pontos e particulas no hardware." " (point-width 5)\n (hint-points)\n (build-sphere 10 10) ; faz uma esfera com pontos grossos\n")) ("blend-mode" ("src dst" "void" "Ajusta o modo de mistura do estado de desenho corrente, ou da primitiva pega. Esse é o modo que o alpha é composto no superficie renderizada." " ; list out all the possible blendmodes\n \n (define src-blend (vector 'zero 'one 'dst-color 'one-minus-dst-color 'src-alpha\n                     'one-minus-src-alpha 'dst-alpha 'one-minus-dst-alpha\n                     'src-alpha-saturate))\n \n (define dst-blend (vector 'zero 'one 'dst-color 'one-minus-dst-color 'src-alpha\n                     'one-minus-src-alpha 'dst-alpha 'one-minus-dst-alpha))\n \n ; picks a random element\n (define (pick-rnd-item l)\n     (vector-ref l (random (vector-length l))))\n \n ; make lots of random spheres\n (define (rnd-sphere n)\n     (push)\n     (hint-depth-sort)\n     (opacity 0.5)\n     (colour (vector (flxrnd) (flxrnd) (flxrnd)))\n \n     ; set a random blendmode\n     (blend-mode (pick-rnd-item src-blend) (pick-rnd-item dst-blend))\n \n     (translate (vector (flxrnd) (flxrnd) (flxrnd)))\n     (scale (vector 0.1 0.1 0.1))\n     (build-sphere 10 10)\n     (pop)\n     (if (zero? n)\n         0\n         (rnd-sphere (- n 1))))\n \n (clear)\n (clear-colour (vector 0.5 0.5 0.5))\n (rnd-sphere 100)\n")) ("hint-solid" ("" "void" "Ajusta as dicas pra renderizar como solid no estado de desenho corrente, ou a primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-solid) ; esse é o estilo de render original então não deve\n              ; muito estimulante\n (build-cube) ; faz um cubo renderizado sólido \n")) ("hint-wire" ("" "void" "Ajusta o render para wireframe no modo de estado corrente, ou da primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-wire)\n (build-cube) ; faz um cubo em wireframe \n")) ("hint-normal" ("" "void" "Ajusta as dicas de render para fazer aparecer normais no estado de desenho corrente, ou da primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-normal)\n (build-cube) ; mostra as normais do cubo\n")) ("hint-points" ("" "void" "Ajusta as dicas para aparecer pontos no estado de desenho corrente, ou da primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-points)\n (build-cube) ; mostra os pontos dos vertices deste cubo\n")) ("hint-anti-alias" ("" "void" "Ajusta as dicas de render para anti-alias no estado de desenho atual, ou da primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-anti-alias)\n (build-cube) ; mostra um cubo macio\n")) ("hint-unlit" ("" "void" "Ajusta as dicas de render para luzes desligadas no estado de desenho corrente, ou da primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-unlit)\n (build-cube) ; mostra um cubo sem iluminação\n")) ("hint-vertcols" ("" "void" "Ajusta as dicas de render pra usar cores de vértices no estado de desenho corrente, ou da primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas. Cores de vértices modificam o estado atual de (colour)." " (hint-vertcols)\n (define mycube (build-cube)) ; faz um cubo com vertcols ativado\n (grab mycube)\n (pdata-set \"c\" 0 (vector 0 1 0)) ; ajusta a cor do primeiro vert\n                                  ; pra verde\n (ungrab)\n")) ("hint-box" ("" "void" "Ajusta as dicas de render para mostrar a caixa envolvente no estado de desenho corrente, ou a primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-box)\n (build-sphere 10 10) ; faz uma esfera com a bounding box\n")) ("hint-multitex" ("" "void" "Ajusta as dicas de render para usar multiplas texturas no estado de desenho corrente. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-multitexture)\n (multitexture 0 (load-texture \"tex1.png\"))\n (multitexture 1 (load-texture \"tex2.png\"))\n (build-sphere 10 10) ; faz uma esfera com texturas sobrepostas\n")) ("hint-none" ("" "void" "Limpa as dicas de render no estado de desenho corrente, ou da primitiva pega. Isso permite que você se livre do estilo sólido default, mas também significa que voce pode ligar e desligar dicas sem usar push ou pop" " (hint-none)\n (hint-wire)\n (build-cube) ; faz um cubo vísivel apenas em wireframe\n")) ("hint-origin" ("" "void" "Ajusta as dicas de render para mostrar a origem espacial do objeto no estado de desenho corrente, ou da primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-origin)\n (build-sphere 10 10) ; make a sphere with the origin displayed\n")) ("hint-cast-shadow" ("" "void" "(nota: não implementado ainda) Ajusta as dicas de render para fazer sombra para o estado de desenho atual, ou a primitiva pega. Dicas de render podem mudar a forma que as primitivas são renderizadas, mas pode ter efeitos diferentes- ou nenhum efeito em certas primitivas, portanto o nome dica." " (hint-origin)\n (build-sphere 10 10) ; make a sphere with the origin displayed\n")) ("hint-depth-sort" ("" "void" "Ajusta as dicas de render para separar em profundidade o estado corrente, ou a primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." " (hint-depth-sort)\n (build-sphere 10 10) \n")) ("hint-ignore-depth" ("" "void" "Ajusta as dicas de render para ignorar tested de profundidade no estado corrente de desenho, ou da primitiva pega. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas. Essa capacidade é util para renderizar objetos transparentes, já que ela significa que objetos vão aparecer atrás de outros já renderizados." "")) ("hint-lazy-parent" ("" "void" "Ajusta as dicas de render para prevenir esta primitiva de passar suas transformações a um filho. Dicas de render mudam a forma como as primitivas são renderizadas, mas podem ter efeitos diferentes - ou nenhum efeito em certas primitivas portanto o nome dicas." "")) ("texture" ("número-id-textura" "void" "Ajusta a textura do estado corrente, ou da primitiva pega. Ids de textura podem ser geradas pela função load-texture." " (texture (load-texture \"mytexture.png\"))\n (build-sphere 10 10) ; faz uma textura com mytexture.png\n")) ("multitexture" ("número-textureunit número-id-primitiva" "void" "Ajusta a textura do estado corrente de desenho, ou da primitiva pega da mesma forma que a função textura, mas permite que você especifique a unidade de textura (0-7) para aplicar-lá.Multitextura permite você aplicar diferentes texturas e coordenadas ao mesmo objeto de uma vez. Unidade de textura 0 é o padrão (que usa a pdata \"t\" para suas coordenadas) unidade de textura n olha pela pdata \"tn\" - ie multitexture 1 olha por \"t1\". Você precisa adicionar estas você mesmo usando (pdata-add) ou (pdata-copy). Multitexturas é útil quando a textura contém alpha, já que elas podem ser sobrepostas, por exemplo adesivos colocados em texturas de fundo. Nota: Fluxus precisa ser compilado usando scons MULTITEXTURE=1 para ativar essa capacidade." " (define obj (build-sphere 10 10)) ; make a sphere \n (grab obj)\n (multitexture 0 (load-texture \"mytextureA.png\")) \n (multitexture 1 (load-texture \"mytextureB.png\"))\n (pdata-add \"t1\" \"v\")   ; make some texture coords for texture B\n (pdata-copy \"t\" \"t1\")  ; copy them from the default texture coords\n (ungrab)\n")) ("print-scene-graph" ("" "void" "Mostra o scene graph corrente, útil para debug." " (print-scene-graph) ; exciting...\n")) ("hide" ("número-escondido" "void" "Ajusta o estado escondido para a primitiva pega (tbm afeta todas as primitivas filhas). Primitivas escondidas podem ser tratadas normalmente em todas as formas - elas apenas não serão renderizadas." " (define obj (build-cube))\n (grab obj)\n (hide 1) ; hide this cube\n (ungrab)\n")) ("selectable" ("número-selecionável" "void" "Ajusta se a primitiva pega pode ser selecionada ou não." " (define obj (build-cube))\n (grab obj)\n (selectable 0) ; agora ela nao vai ser \"vista\", quando chamar select\n (ungrab)\n")) ("shader" ("vertexprograma-string fragmentprogram-string" "void" "Abre, compila e ajusta o pard de GLSL hardware shader para o estado de desenho atual, ou a primitiva pega. Requer OpenGL 2. Os dados uniformes do shader podem ser controlados via shader-set! e todas as pdatas são enviadas como um atributo por vértice ao shader." " (push)\n ; assign the shaders to the surface\n (shader \"simplevert.glsl\" \"simplefrag.glsl\")\n (define s (build-sphere 20 20))\n (pop)\n \n (grab s)\n ; add and set the pdata - this is then picked up in the vertex shader \n ; as an input attribute called \"testcol\"\n (pdata-add \"testcol\" \"v\")\n (set-cols (pdata-size))\n (ungrab)\n \n (define (animate)\n     (grab s)\n \t; animate the deformamount uniform input parameter \n     (shader-set! (list \"deformamount\" (cos (time))))\n     (ungrab))\n \n (every-frame (animate))\n")) ("shader-set!" ("lista-argumentos" "void" "Ajusta os parâmetros do shader uniforme para o shader GLSL. A lista contém valores pares simbolos-strings, que relacionam os parâmetros de shader correspondentes nomes e valores" " (push)\n ; assign the shaders to the surface\n (shader \"simplevert.glsl\" \"simplefrag.glsl\")\n (define s (build-sphere 20 20))\n (pop)\n \n (grab s)\n ; add and set the pdata - this is then picked up in the vertex shader \n ; as an input attribute called \"testcol\"\n (pdata-add \"testcol\" \"v\")\n (set-cols (pdata-size))\n (ungrab)\n \n (define (animate)\n     (grab s)\n \t; animate the deformamount uniform input parameter \n     (shader-set! (list \"deformamount\" (cos (time))))\n     (ungrab))\n \n (every-frame (animate))\n"))))) ("matemática" ("Estas funções sao optimizadas para gráficos em 3d, e a colisão entre ciência da computação e matemática é aparente aqui, então vetores representando \"vectors\" são nesse contexto tidos como 3 elementos em tamanho, quaternions são vetores de tamanho 4, e matrizes são vetores de 16 elementos." "" (("vmul" ("vetor número" "vetor resultante" "Multiplica um vetor por um número." " (vmul (vector 1 2 3) 2)\n")) ("vadd" ("vetor vetor" "vetor resultante" "Adiciona dois vetores, um ao outro." " (vadd (vector 1 2 3) (vector 1 2 3))\n")) ("vsub" ("vetor vetor" "vetor resultante" "Subtrai um vetor de outro." " (vsub (vector 1 2 3) (vector 1 2 3))\n")) ("vdiv" ("vetor número" "vetor resultante" "Divide um vetor por um número" " (vdiv (vector 1 2 3) 2)\n")) ("vtransform" ("vetor matriz" "vetor resultante" "Multiplica (transforma) um vetor por uma matriz." " (vtransform (vector 0 1 0) (mrotate (vector 90 0 0)))\n")) ("vtransform-rot" ("vetor matriz" "vetor resultante" "Multiplica (transforma) um vetor por uma matriz, mas deixa de fora a parte de translação. Para ser usado em operações involvendo normais." " (vtransform-rot (vector 0 1 0) (mrotate (vector 90 0 0)))\n")) ("vnormalise" ("vetor" "vetor resultante" "Retorna a forma normalisada do vetor (length=1)" " (vtransform-rot (vector 0 1 0) (mrotate (vector 90 0 0)))\n")) ("vdot" ("vetor vetor" "número resultante" "Retorna o produto multiplicado de dois vetores." " (vdot (vector 0 1 0) (vector 1 0 0))\n")) ("vmag" ("vetor" "número resultante" "Retorna a magnitude, ou alcance do vetor" " (vmag (vector 0 1 1))\n")) ("vdist" ("vetor vetor" "número resultante" "Tratando os vetores como pontos, retorna a distancia entre eles." " (vdist (vector 100 100 0) (vector 0 0 100))\n")) ("vcross" ("vetor vetor" "vetor resultante" "Retorna o produto cruzado entre dois vetores, resultando em um vetor que é perpendicular aos cruzados." " (vcross (vector 100 100 0) (vector 0 0 100)) \n")) ("mmul" ("vetor-matriz vetor-matriz" "vetor-matriz" "Multiplica duas matrizes." " (vmul (mtranslate (vector 1 0 0)) (mrotate (vector 0 90 0)))\n")) ("madd" ("vetor-matriz vetor-matriz" "vetor-matriz" "Adiciona duas matrizes." " (vadd (mtranslate (vector 1 0 0)) (mrotate (vector 0 90 0)))\n")) ("msub" ("vetor-matriz vetor-matriz" "vetor-matriz" "Subtrai uma matriz de outra." " (vsub (mtranslate (vector 1 0 0)) (mrotate (vector 0 90 0)))\n")) ("mdiv" ("vetor-matriz vetor-matriz" "vetor-matriz" "Divide uma matriz por outra." " (vdiv (mtranslate (vector 1 0 0)) (mrotate (vector 0 90 0)))\n")) ("mident" ("" "vetor-matriz" "Retorna a matriz identidade" " (mident)\n")) ("mtranslate" ("vetor" "vetor-matriz" "Retorna uma matriz representando a tranformação(translação) especificada." " (mtransform (vector 100 0 0))\n")) ("mrotate" ("vetor" "vetor-matriz" "Retorna uma matriz representando a rotação especificada. Aceita um vetor de angulos euler, ou um quatérnio." " (mrotate (vector 0 45 0))\n")) ("mscale" ("vetor" "vetor-matriz" "Retorna uma matriz representando a escalagem especificada." " (mscale (vector 0.5 2 0.5))\n")) ("mtranspose" ("vetor-matriz" "vetor matriz" "Retorna a transposta do vetor de entrada" " (mtranspose (mident))\n")) ("minverse" ("vetor-matriz" "vetor-matriz" "Retorna o inverso do vetor de entrada." " (minverse (mscale (vector 0.5 2 0.5)))\n")) ("maim" ("vetor-mira vetor-acima" "vetor-matriz" "Retorna uma matriz representando uma rotação de mira de forma que o eixo X aponta pra baixo da direção de mira, e o eixo y aponta pra cima do vetor de cima. Provavelmente sofre do Gimbal Lock." " (maim (vector 0 0 1) (vector 0 1 0))\n")) ("qaxisangle" ("vetor-eixo angulo" "vetor-quaternion" "Retorna o quatérnio representando o ângulo de rotação sobre o eixo especificado." " (qaxisangle (vector 0 1 0) 45)\n")) ("qmul" ("vetor-quatérnio vetor-quatérnio" "vetor-quatérnio" "Multiplica um quatérnio por outro." " (qmul (qaxisangle (vector 0 1 0) 45) (qaxisangle (vector 0 0 1) 180))\n")) ("qnormalise" ("vetor-quatérnio" "vetor-quatérnio" "Normalisa um quatérnio" " (qnormalise (qaxisangle (vector 0 19 0) 45))\n")) ("qtomatrix" ("vetor-quatérnio" "vetor-matriz" "Converte um quatérnio em uma matriz de rotação" " (qtomatrix (qaxisangle (vector 0 1 0) 45))\n")) ("qconjugate" ("vetor-quatérnio" "vetor-quatérnio" "Conjuga um quatérnio" " (qconjugate (qaxisangle (vector 0 1 0) 45))\n"))))) ("primitive-data" ("Dados primitivos (pdata para diminuir) é o nome fluxus' para dados que formam primitivas. Em primitivas poligonais isto significa informação dos vértices, em primitivas de partículas isto corresponde a informação da partícula, em primitivas NURBS são os vértices de controle. Acesso a pdata dá a você a habilidade de usar primitivas que de outra forma não seria tão interessante, e deformar e fazer outras primitivas resultando em modelos muito mais detalhados e animações. Você pode também adicionar sua própria pdata, que é tratado exatamente como os tipos já existentes. Dados primitivos são nomeados por strings de tipo, os nomes de qual depende a ordem da primitiva. Todos os comandos Pdata operam na primitiva atualmente pega [grabbed]." "" (("pdata-ref" ("string-tipo número-index" "vetor-valor/cor/matriz/número" "Retorna o elemento pdata correspondente." " (pdata-ref \"p\" 1)\n")) ("pdata-set!" ("string-tipo número-index vetor-valor/cor/matriz/número" "void" "Escreve ao elemento pdata correspondente." " (pdata-set! \"p\" 1 (vector 0 100 0))\n")) ("pdata-add" ("string-tipo nome-string" "void" "Adiciona um novo usuario de disposição ao pdata. Tipo é um dos sequintes \"v\":vector, \"c\":colour, \"f\":float ou \"m\":matrix." " (pdata-add \"v\" \"mydata\")\n (pdata-set \"mydata\" 0 (vector 1 2 3))\n")) ("pdata-op" ("string-nomefunc string-nomepdata operador" "void" "Esta é uma função experimental que permite a você fazer operações na pdata muito rapidamente, para constar adicionar elemento por elemento de uma array de pdata para outra. Você pode implementar isto em scheme como um loop sobre cada elemento, mas isto é devagar já que o interpretador está fazendo todo o trabalho. É muito mais rápido se você puder usar um pdata-op já que a mesma operação vai ser apenas uma chamada à scheme." " (pdata-op \"+\" \"mydata\" (vector 1 2 3))  adiciona um vetor a todos os vetores pdata\n (pdata-op \"+\" \"mydata\" \"myotherdata\")  adiciona dois vetores pdata elemento por elemento\n (pdata-op \"*\" \"mydata\" (vector 1 2 3))  multiplica um vetor a todos os vetores pdata\n (pdata-op \"*\" \"mydata\" \"myotherdata\")  multiplica dois vetores pdata elemento por elemento\n (pdata-op \"closest\" \"p\" (vector 100 0 0))  retorna posição do vertice mais perto a este ponto\n (pdata-op \"sin\" \"mydata\" \"myotherdata\")  seno de um pdata float a outro\n (pdata-op \"cos\" \"mydata\" \"myotherdata\")  cosine of one float pdata to another\n")) ("pdata-copy" ("string-pdata-de string-pdata-para" "void" "Copia o conteúdo de uma array pdata para outra. As arrays tem que ser do mesmo tipo." " (pdata-copy \"p\" \"mydata\") ; copia as posições de vértices para uma array do usuário\n")) ("pdata-size" ("" "número-contador" "Retorna o tamanho das arrays pdata (elas precisam ser todas a mesma). Isto é principal para iterar através das arrays." " (define (mashup n)\n     (pdata-set \"p\" n (vector (flxrnd) (flxrnd) (flxrnd))) ; randomise the vertex position\n     (if (zero? n)\n         0\n         (mashup (- n 1)))) ; loops till n is 0\n\n (define shape (build-sphere 10 10))\n (grab shape)\n (mashup (pdata-size)) ; randomise verts on currently grabbed primitive\n (ungrab)\n")) ("finalise" ("" "void" "Não faz nada mais, precisa ser removido :)" "")) ("recalc-normals" ("número-macioounão" "void" "Para primitivas poligonais apenas. Olha a posição dos vértices e cálcula as normais da luz pra você automaticamente. Chame com \"1\" para normais macias, \"0\" para normais facetadas." " (define shape (build-sphere 10 10)) ; build a sphere (which is smooth by default)\n (grab shape)\n (recalc-normals 0) ; make the sphere faceted\n (ungrab)\n"))))) ("physics" ("O sistema de física em fluxus é baseado na biblioteca ODE, que permite a você adicionar propriedades físicas a objetos e colocá-los em movimento. Já que ODE foi projetada para simulações em corpos rígidos, estruturas são descritas em termos de objetos, dobras e forças. Uma explicação muito mais compreensiva destes conceitos pode ser encontrada na documentação de ODE, que você provavelmente baixou se teve que compilar fluxus, ou pode ser encontrado em @url{http://ode.org/ode-docs.html} Para ajudar com a depuração de dobras, tente chamar (render-physics) em cada frame, o que vai renderizar locators mostrando a você posições e eixos de dobras que possuem informação posicional." "" (("collisions" ("on/off-número" "void" "Habilita ou desabilita detecção de colisão. Padrão é desligado." " (collisions 1)\n")) ("ground-plane" ("vetor-plano número-offset" "void" "Cria um plano passivo infinito pra usar como 'chão'." " (ground-plane (vector 0 1 0) 0)\n")) ("active-box" ("número-id-primitiva" "void" "Permite que o objeto seja afetado pelo sistema físico, usando uma caixa como um volume limite. Como um objeto ativo, ele vai ser transformado por ODE. Nota: rotações só funcionam corretamente se você específicar suas escalagens primeiro, depois rotacionar (translação não importa) basicamente, ode não pode lidar com tranformações cortadas." " (define mycube (build-cube))\n (active-box mycube)\n")) ("active-cylinder" ("número-id-primitiva" "void" "Permite que o objeto seja afetado pelo sistema físico, usando um cilindro como um volume limite. Como um objeto ativo, ele vai ser transformado por ODE. Nota: rotações só funcionam corretamente se você específicar suas escalagens primeiro, depois rotacionar (translação não importa) basicamente, ode não pode lidar com tranformações cortadas." " (define mycube (build-cube))\n (active-cylinder mycube)\n")) ("active-sphere" ("número-id-primitiva" "void" "Permite que o objeto seja afetado pelo sistema físico, usando uma esfera como um volume limite. Como um objeto ativo, ele vai ser transformado por ODE. Nota: rotações só funcionam corretamente se você específicar suas escalagens primeiro, depois rotacionar (translação não importa) basicamente, ode não pode lidar com tranformações cortadas." " (define mycube (build-cube))\n (active-sphere mycube)\n")) ("passive-box" ("número-id-primitiva" "void" "Permite que o objeto seja resolvido pelo sistema de física, usando uma caixa como o limite. Como um objeto passivo, objetos ativos vão colidir com ele, mas este não vai ser transformado. Nota: rotações só funcionam corretamente se você especificar suas transformaçoes de escala primeiro, depois rotacionar (translação não importa) basicamente, ode não pode lidar com transformações cortadas." " (define mycube (build-cube))\n (passive-box mycube)\n")) ("passive-cylinder" ("número-id-primitiva" "void" "Permite que o objeto seja resolvido pelo sistema de física, usando um cilindro como o limite. Como um objeto passivo, objetos ativos vão colidir com ele, mas este não vai ser transformado. Nota: rotações só funcionam corretamente se você especificar suas transformaçoes de escala primeiro, depois rotacionar (translação não importa) basicamente, ode não pode lidar com transformações cortadas." " (define mycube (build-cube))\n (passive-cylinder mycube)\n")) ("passive-sphere" ("número-id-primitiva" "void" "Permite que o objeto seja resolvido pelo sistema de física, usando uma esfera como o limite. Como um objeto passivo, objetos ativos vão colidir com ele, mas este não vai ser transformado. Nota: rotações só funcionam corretamente se você especificar suas transformaçoes de escala primeiro, depois rotacionar (translação não importa) basicamente, ode não pode lidar com transformações cortadas." " (define mycube (build-cube))\n (passive-sphere mycube)\n")) ("surface-params" ("slip1-número slip2-número softerp-número softcfm-número" "void" "Ajusta alguns parâmetros globais de superfície que afetam o atrito e reflexão. Veja seção 7.3.7 dos docs de ODE para uma explicação desses parâmetros." " (surface-params 0.1 0.1 0.1 0.1)\n")) ("build-balljoint" ("número-id-primitiva número-id-primitiva vetor-eixo" "void" "Cria um balljoint para conectar dois objetos (veja os docs de ode para uma descrição detalhada das diferenças entre os tipos de dobras). ODE considera as juntas(joints) serem uma restrição imposta entre dois objetos. Quando criando uma junção, é importante ter as duas primitivas sendo juntas na posição desejada antes de criar uma junção. Junções podem ser criadas, modificadas e indexadas de uma maneira similar a outras primitivas." " (build-balljoint shape1 shape2 (vector 0 1 0)) \n")) ("build-fixedjoint" ("número-id-primitiva" "void" "Cria uma junção para conectar um objeto ao ambiente global. Isto trava o objeto no lugar." " (build-fixedjoint shape)\n")) ("build-hingejoint" ("numero-id-primitiva1 número-id-primitiva2 vetor-ancora vetor eixo" "void" "Cria uma junção circular para conectar dois objetos ( veja os docs de ODE para uma descrição detalhada entre tipos de junções). ODE considera as juntas(joints) serem uma restrição imposta entre dois objetos. Quando criando uma junção, é importante ter as duas primitivas sendo juntas na posição desejada antes de criar uma junção. Junções podem ser criadas, modificadas e indexadas de uma maneira similar a outras primitivas." " (build-hingejoint shape1 shape2 (vector 0 1 0) (vector 0 1 0)) \n")) ("build-sliderjoint" ("número-id-primitiva1 número-id-primitiva2 vetor-eixo" "hingeid-number" "Cria uma junção deslizante entre dois objetos (veja a documentação de ODE para uma descrição detalhada das diferenças entre os tipos de conexões). ODE considera as juntas(joints) serem uma restrição imposta entre dois objetos. Quando criando uma junção, é importante ter as duas primitivas sendo juntas na posição desejada antes de criar uma junção. Junções podem ser criadas, modificadas e indexadas de uma maneira similar a outras primitivas." " (build-sliderjoint shape1 shape2 (vector 0 1 0)) \n")) ("build-hinge2joint" ("número-id-primitiva1 número-id-primitiva2 vetor-ancora vetor-eixo1 vetor-eixo2" "numero-id-dobradiça(hinge)" "Cria uma dobradiça de junções para conectar dois objetos (veja a documentação de ODE para uma descrição detalhada das diferenças entre os tipos de conexões). ODE considera as juntas(joints) serem uma restrição imposta entre dois objetos. Quando criando uma junção, é importante ter as duas primitivas sendo juntas na posição desejada antes de criar uma junção. Junções podem ser criadas, modificadas e indexadas de uma maneira similar a outras primitivas." " (build-hinge2joint shape1 shape2 (vector 0 100 0) (vector 0 1 0) (vector 0 1 0))\n")) ("build-amotorjoint" ("número-id-primitiva1 número-id-primitiva2 vetor-eixo" "número-id-dobradiça(hinge)" "Cria uma junção com mobilidade angular para conectar dois objetos (veja a documentação de ODE para uma descrição detalhada das diferenças entre os tipos de conexões). ODE considera as juntas(joints) serem uma restrição imposta entre dois objetos. Quando criando uma junção, é importante ter as duas primitivas sendo juntas na posição desejada antes de criar uma junção. Junções podem ser criadas, modificadas e indexadas de uma maneira similar a outras primitivas." " (build-amotorjoint shape1 shape2 (vector 0 1 0))\n")) ("joint-param" ("número-id-junção param-string número-valor" "número-id-dobradiça(hinge)" "Ajusta o parâmetro da junção para uma junta onde param é um dos seguintes: \"HiStop\", \"Vel\", \"FMax\", \"FudgeFactor\", \"Bounce, \"CFM\", \"StopERP\", \"StopCFM\", \"SuspensionERP\", \"SuspensionCFM\", \"Vel2\", \"FMax2\". Veja Seção 7.5.1 da documentação de ODE para uma explicação de cada um desses parâmetros, e a qual tipos de junções eles aplicam." " (joint-param joint \"Vel\" 0.1)\n")) ("joint-angles" ("jointid-number número-angulo número-vel" "void" "Ajusta um novo ângulo para este joint, com uma velocidade dada para chegar lá." " (joint-angle joint 90 0.1)\n")) ("set-max-physical" ("número-max" "void" "Ajusta o máximo número de objetos que o sistema de física pode lidar. Quando o máximo nível foi alcançado os objetos mais antigos são automaticamente destroidos." " (set-max-physical 100)\n")) ("set-mass" ("número-id-primitiva número-massa" "void" "Ajusta a massa de um objeto ativo no sistema de física" " (set-mass myshape 100)\n")) ("gravity" ("vetor-gravidade" "void" "Ajusta a força e direção da gravidade." " (gravity (vector 0 -1 0))\n")) ("kick" ("número-id-primitiva vetor-chute" "void" "Aplica força de translação ao objeto." " (kick myshape (vector 0 1 0))\n")) ("twist" ("número-id-primitiva vetor-giro" "void" "Aplica força rotacional ao objeto" " (twist myshape (vector 2 0 0))\n")) ("has-collided" ("número-id-primitiva" "void" "Retorna verdadeiro se a primitiva pega colidiu no ultimo frame." " (if (has-collided myshape) (display \"bang!\"))\n"))))) ("primitivas" ("Primitivas são objetos que você pode renderizar. Não há muito mais coisas numa cena do fluxus, exceto luzes, uma camera e muitas primitivas." "" (("build-cube" ("" "número-de-id-da-primitiva" "Um simples cubo, mapeamento de textura por face. @image{images/cube}" " (define mynewcube (build-cube))\n")) ("build-polygons" ("número-de-vértices tipo-de-número" "número-de-id-primitiva" "Cónstroi uma primitiva de polígono cru com tamanho de vértices (tudo é inicialmente tido como zero). Tipo é um número que referencia a forma as quais os vértices são interpretados para construir os poligonos, e podem ser os seguintes: 0=TRISTRIP, 1=QUADS, 2=TRILIST, 3=TRIFAN, 4=POLYGON." " (define mynewshape (build-polygons 100 'triangle-strip))\n")) ("build-sphere" ("número-de-cortes-horizontais número-de-cortes-verticais" "número-id-primitiva" "Uma esfera com a resolução especificada, a textura mapeada no estilo normal \"world map\". @image{images/sphere}" " (define mynewshape (build-sphere 10 10))\n")) ("build-plane" ("" "número-id-primitiva" "Um único plano quadrado, mapeado de 0->1 em ambas as dimensões. @image{images/plane}" " (define mynewshape (build-plane))\n")) ("build-seg-plane" ("número-de-vértices-x número-de-vértices-y" "número-id-primitiva" "Um plano poligonal tesselado, mapeado de 0->1 em ambas dimensões." " (define mynewshape (build-plane))\n")) ("build-cylinder" ("h-segmentos r-segmentos" "número-id-primitiva" "Um cilindro coberto, textura mapeada em volta, e mal mapeada em volta do fim. @image{images/cylinder}" " (define mynewshape (build-cylinder 10 10))\n")) ("build-line" ("numpoints-número" "número-id-primitiva" "Cónstroi uma linha consistindo de numpoints pontos. A geometria aponta para a câmera constantemente e a textura é mapeada de forma que se alonga na linha do início ao fim. Você usa as funções pdata para editar as posições e largura das linhas. Se usado iluminado, as normais são falseadas para aproximar uma seção circular cruzada. Adicionalmente, se renderização sólida for limpa com (hint-none) e (hint-wire) ativado, uma rápida linha constante vai ser desenhada - largura específicada pelo comando (line-width). @image{images/line}" " (define mynewshape (build-line 10))\n")) ("build-text" ("string-texto" "número-id-primitiva" "Constrói uma sequencia de planos, mapeados de forma que uma textura de fonte possa ser usada para visualização. Pode vir a ser útil para coisas mais abstratas. A fonte é assumida como não proporcional - tem um exemplo de fonte acompanhando o fluxus. @image{images/text} Ok, so this isn't a very good font texture :)" " (texture (texture-load \"font.png\"))\n (define mynewshape (build-text \"hello\"))\n")) ("build-nurbs-sphere" ("h-segmentos r-segmentos" "número-id-primitiva" "Constrói uma esfera nurbs tesselada, mapead da mesma forma que a esfera poligonal. @image{images/nurbs-sphere}" " (define mynewshape (build-nurbs-sphere 10 10))\n")) ("build-nurbs-plane" ("h-segmento r-segmento" "número-id-primitiva" "Constrói um plano nurbs tesselado, mapeado na direção uv. @image{images/nurbs-plane}" " (define mynewshape (build-nurbs-plane 10 10))\n")) ("build-particles" ("número-contagem" "número-id-primitiva" "Cónstroi uma primitiva de partículas contendo num pontos, tudo inicialmente aplicado à origem. Você usa as funções pdata para editar as posições, cores e tamanhos. Partículas vêm em dois tipos, sprites apontando pra câmera, que são o padrão, podem ser texturizadas e escaladas individualmente; e pontuais (quando hint-points está aplicado), que não podem ser texturizadas mas são muito mais rápidas de renderizar, já que elas são pontos gl suportados pelo hardware. Por defeito essas partículas pontuais são quadradas, ligue hint-anti-alias para faze-las circulares. @image{images/sprites}" " (define mynewshape (build-particles 100))\n")) ("build-locator" ("" "número-id-primitiva" "Um locator é uma primitiva vazia, útil para parentesco (quando você não quer ter o objeto pai visivel). Essa primitiva só pode ser visualizada com (hint-origin) para mostrar sua origem de transformação local." " (define mynewshape (build-locator))\n")) ("build-pixels" ("número-largura número-altura" "número-id-primitiva" "Faz uma nova primitiva pixel. Uma primitiva pixel é usada pra fazer texturas procedurais, que podem então ser aplicadas em outras primitivas. Por essa razão, primitivas pixel não vão ser renderizadas muito, mas você pode renderizar elas para visualizar a texturas em um plano." " (define mynewshape (build-pixels 100 100))\n")) ("pixels-upload" ("" "void" "Traz os dados da textura, você precisa chamar isto quando você finalizou escrever ao pixelprim, e enquanto ele está \"grabbed\"." " (define mynewshape (build-pixels 100 100))\n (pixels-upload mynewshape)\n")) ("pixels->texture" ("número-id-pixelprim" "número-id-textura" "Retorna uma textura que você pode usar exatamente igual uma que foi carregada normalmente." " (define mynewshape (build-pixels 100 100))\n (upload-pixels mynewshape)\n (texture (pixels->texture mynewshape))\n")) ("build-blobby" ("número-influencias vecsubdvision vecfronteira" "número-id-primitiva" "Primitivas blobby no fluxus são uma representação de superfície implicita de alto nível que é definida usando influências no espaço em 3 dimensões. Estas influências são então somadas, e um valor particular é \"malheado\" (usando o algorítmo dos cubos marchando) para formar uma superfície macia. Estas influências podem ser animadas, e a superfície macia mexe e deforma para adaptar, dando a primitiva seu nome blobby. build-blobby retorna uma nova primitiva blobby. número-influências é o número de \"blobs\", Subdivisão permite a você controlar a resolução da superfície em cada dimensão, enquanto vecfronteira ajusta a área fronteriça da primitiva em espaço de objeto local. A malha não vai ser calculada fora desta área limite. Influências de cores e posições precisam ser ajustadas usando pdata-set." " (define mynewshape (build-blobby 7 (vector 30 30 30) (vector 3 3 3)))\n")) ("blobby->poly" ("número-id-blobbyprimitiva" "número-id-primitivapoly" "Converte a malha de uma primitiva blobby em uma primitiva poligonal de lista de triângulos." " (define mynewshape (blobby->poly myblobby))\n")) ("draw-instance" ("número-id-primitiva" "void" "Copia um modo retido da primitiva e desenha ela no estado corrente como um no modo imediato." " (define mynewshape (build-cube))\n (colour (vector 1 0 0))\n (draw-instance mynewshape) ; draws a copy of mynewshape\n")) ("draw-cube" ("" "void" "Desenha um cubo no estado imediato corrente." " (define (render)\n     (draw-cube))\n (every-frame (render))\n")) ("draw-plane" ("" "void" "Desenha um plano no estado corrente em modo imediato" " (define (render)\n     (draw-plane))\n (every-frame (render))\n")) ("draw-sphere" ("" "void" "Desenha uma esfera no estado corrente em modo imediato." " (define (render)\n     (draw-sphere))\n (every-frame (render))\n")) ("draw-cylinder" ("" "void" "Desenha um cilindro no estado corrente em modo imediato." " (define (render)\n     (draw-cylinder))\n (every-frame (render))\n")) ("destroy" ("número-id-primitive" "void" "Deleta uma primitiva construída do renderizador." " (define mynewshape (build-sphere 10 10))\n (destroy mynewshape)\n")) ("poly-set-index" ("lista-indexada" "void" "Troca a primitiva por modo indexado,e usa a lista como valores de index para essa primitiva." " (clear)\n (define p (build-polygons 8 1))\n \n (grab p)\n ; setup the vertex data\n (pdata-set \"p\" 0 (vector -1 -1 -1))\n (pdata-set \"p\" 1 (vector  1 -1 -1))\n (pdata-set \"p\" 2 (vector  1 -1  1))\n (pdata-set \"p\" 3 (vector -1 -1  1))\n (pdata-set \"p\" 4 (vector -1  1 -1))\n (pdata-set \"p\" 5 (vector  1  1 -1))\n (pdata-set \"p\" 6 (vector  1  1  1))\n (pdata-set \"p\" 7 (vector -1  1  1))\n \n (hint-wire)\n (hint-unlit)\n \n ; connect the verts together into faces\n (poly-set-index (list 7 6 5 4  5 6 2 1 \n                        4 5 1 0  1 2 3 0\n                        3 7 4 0  6 7 3 2))\n \n (ungrab)\n")) ("poly-convert-to-indexed" ("" "void" "Converte o atual poligono primitivo que está \"grabbed\" de arrays de vértices crus para arrays indexadas. Isto remove vértices duplicados do polígono, fazendo a array de pdata menor, o que aumenta a velocidade do processo." " (define mynewshape (build-sphere 10 10))\n (grab mynewshape)\n (poly-convert-to-indexed)\n (ungrab)\n")) ("build-copy" ("número-primitiva-fonte" "número-id-primitiva" "Retorna uma cópia da primitiva" " (define mynewshape (build-sphere 10 10))\n (define myothernewshape (build-copy mynewshape))\n")) ("make-pfunc" ("string-nome" "número-pfuncid" "Faz uma nova primitiva funcional. Pfuncs variam de propósito geral até operações complexas e especializadas que você pode rodar em primitivas. Todas as pfuncs dividem a mesma interface para controlar e ajustar - pfunc-set! Todos os tipos e argumentos pfunc são como os seguintes: arithmetic Para aplicar aritmética geral para qualquer array pdata. operator string : um dos add sub mul div src string : nome pdata array other string : nome pdata array (opcional) constant float : valor constante (opcional) dst string : nome pdata array genskinweights Gera pesos de skinning - adiciona float pdata chamada \"s1\" -> \"sn\" aonde n é o número de nós no esqueleto - 1 skeleton-root primid-number : a raiz da posebind do esqueleto para skinning sharpness float : um controle de quão afiado o vinco vai ser quando \"skineado\". skinweights->vertcols Uma utilidade para visualizar pesos de skin para debugar. sem argumentos. skinning Skin uma primitiva - deforma ela para seguir os movimentos de um esqueleto. Primitivas que a gente quer usar isto deve conter extra pdata - cópias das posições iniciais das posições dos vértices chamadas \"pref\" e o mesmo para normais, se as normais estão sendo skineadas , chamada \"nref\". skeleton-root primid-number : a primitiva raiz do esqueleto animado bindpose-root primid-number : a raiz primitiva da pose bind do esqueleto skin-normals number : se devemos usar skin nas normais como nas posições" " (define mypfunc (make-pfunc 'arithmetic))\n")) ("pfunc-set!" ("número-id-pfunc lista-argumento" "void" "Ajusta argumentos na função primitiva. Veja a documentação de make-pfunc para todos os argumentos." " (define mypfunc (make-pfunc 'arithmetic))\n (pfunc-set! mypfunc (list 'operator \"add\"\n                           'src \"p\"\n                           'const 0.4\n                           'dst \"p\"))\n")) ("pfunc-run" ("número-id" "void" "Roda uma função primitiva na primitiva atualmente pega." " (define mypfunc (make-pfunc \"arithmetic\"))\n"))))) ("turtle" ("O construtor polygonal Turtle é um modo experimental de construir objetos poligonais usando uma tartaruga do estilo logo no espaço 3d. Em que você pode dirigir a tartaruga ao redor colocando vértices e construindo formas proceduralmente. A tartaruga também pode ser usa para deformar objetos poligonais existentes, fixando ela a objetos que você já criou." "" (("turtle-prim" ("número-tipo" "void" "Inicia a construção de uma nova primitiva poligonal com a tartaruga. O tipo especifica o tipo da face do poligono e é um dos seguintes: 0: triangle strip, 1: quad list, 2: triangle list, 3: triangle fan, 4: general polygon" " (turtle-prim 0)\n")) ("turtle-vert" ("" "void" "Cria um novo vértice na posição atual, ou ajusta o vértice atual se o construtor tartaruga estiver fixado." " (turtle-vert)\n")) ("turtle-build" ("" "void" "Constrói o objeto com a lista de vértices definida e dá isto ao renderizador. Não tem efeito se o construtor tartaruga estive fixado a uma primitiva." " (define mynewshape (turtle-build))\n")) ("turtle-move" ("número-distância" "void" "Move a tartaruga pra frente na sua orientação atual." " (turtle-move 1)\n")) ("turtle-push" ("" "void" "O construtor tartaruga tem sua própria pilha de transformações, Push lembra a posição atual e orientação." " (turtle-push)\n")) ("turtle-pop" ("" "void" "O construtor tartaruga tem sua própria pilha de transformações. Pop esquece a posição atual e orientação, e vai de volta para o estado do último push." " (turtle-pop)\n")) ("turtle-turn" ("vetor-rotação" "void" "Rotaciona a orientação da tartaruga com o ângulos euler fornecidos (rotações em x, y e z)" " (turtle-turn (vector 45 0 0))\n")) ("turtle-reset" ("" "void" "Reinicia a atual posição e rotação da tartaruga para a origem." " (turtle-reset)\n")) ("turtle-attach" ("número-id-primitiva" "void" "Anexa a tartaruga a uma primitiva poligonal existente. Isso significa que você será capaz de deformar pontos de objetos existentes usando o construtor tartaruga." " (define myshape (build-sphere 10 10))\n (turtle-attach myshape)\n")) ("turtle-skip" ("número-contador" "void" "Quando anexado, faz com que a tartaruga pule vértices. Este valor pode ser negativo, o que vai resultar na tartaruga escrevendo para vértices anteriores." " (turtle-skip -1)\n")) ("turtle-position" ("" "void" "Quando anexado, retorna o índice atual da pdata que a tartaruga esta escrevendo para." " (display (turtle-position))(newline)\n")) ("turtle-seek" ("número-posição" "void" "Quando anexado, ajusta o índice absoluto da pdata que a tartaruga está escrevendo para." " (turtle-seek 0)\n"))))) ("util-functions" ("Fun????es pr??ticas para fazer a sua vida mais f??cil." "" (("time" ("" "numero do tempo(transcorrido, passado)" "Retorna o numero de segundos (+ fracao) desde meia noite do dia 1 de janeiro de 1970. Essa e a forma mais simples de animacao para seus scripts" " (define (animate)\n     (rotate (vector (sin (time)) 0 0))\n     (draw-cube))\n (every-frame (animate))    \n")) ("delta" ("" "numero do tempo(transcorrido, passado)" "Tempo em segundos desde o ultimo frame. Usado pra fazer animacoes com taxa de frames independendente(constante)." " (define (animate)\n     (rotate (* (delta) 10) 0 0) \n     (draw-cube))\n (every-frame (animate))    \n")) ("flxrnd" ("" "n??mero rand??mico" "Retorna um n??mero rand??mico entre 0 e 1." " (define (animate)\n     (colour (flxrnd) (flxrnd) (flxrnd)) \n     (draw-cube))\n (every-frame (animate))\n")) ("flxseed" ("n??mero da semente" "void" "Alimenta o gerador de n??mero rand??mico de forma que n??s consigamos a mesma sequ??ncia." " (define (animate)\n     (colour (flxrnd) (flxrnd) (flxrnd)) \n     (draw-cube))\n (flxseed 10) \n (every-frame (animate)) ; a mesma sequ??ncia de cores ser?? gerada\n")) ("searchpaths" ("lista de diret??rios" "void" "Arranja um lista de diret??rios de busca para procurar por arquivos relacionado ao fluxus, como texturas, shaders etc. Os diret??rios ser??o procurados em ordem a cada vez." " (searchpaths (list \"/path/to/my/textures\" \"/path/to/my/other/textures\"))\n")) ("fullpath" ("string do nome do arquivo" "fullpath-string" "Busca os diret??rios de procura para o arquivo especificado e retorna a primeira localiza????o que encontra." " (fullpath \"myfile\")\n")) ("framedump" ("nome-do-arquivo" "void" "Salva o buffer OpenGL da frente no disco. L?? a extens??o do arquivo pra decidir qual qual formato ser?? usado pra salvar, \"tif\", \"jpg\" ou \"ppm\" s??o suportados. Esta ?? a forma de frame dumping de baixo nivel, use start-framedump e end-framedump ao inv??s." " (framedump \"picture.jpg\")\n"))))) ("osc" ("OSC significa Open Sound Control, e é um protocolo amplamente usado para passar dados entre aplicações multimidia. Fluxus pode enviar e/ou receber mensagens." " Um exemplo de uso de osc para comunicar entre pd e fluxus.\n Um script fluxus para mover um cubo baseado em mensagens osc\n entrando.\n -- osc.scm\n\n (define value 0)\n\n (define (test)\n     (push)\n     (if (osc-msg \"/zzz\")\n         (set! value (osc 0)))\n     (translate (vector 1 0 value))\n     (draw-cube)\n     (pop))\n \n (osc-source \"6543\")\n (every-frame (test))\n \n --- EOF\n Um patch PD para enviar mensagens de controle ao fluxus:\n --- zzz.pd\n #N canvas 618 417 286 266 10;\n #X obj 58 161 sendOSC;\n #X msg 73 135 connect localhost 6543;\n #X msg 58 82 send /zzz \\$1;\n #X floatatom 58 29 5 0 0 0 - - -;\n #X obj 58 54 / 100;\n #X obj 73 110 loadbang;\n #X connect 1 0 0 0;\n #X connect 2 0 0 0;\n #X connect 3 0 4 0;\n #X connect 4 0 2 0;\n #X connect 5 0 1 0;\n" (("osc-source" ("string-porta" "void" "Inicia o servidor osc, ou muda a porta. Bug conhecido: parece falhar se você usa de volta pra uma mesma porta já usada anteriormente." " (osc-source \"4444\")\t ; listen to port 4444 for osc messages\n")) ("osc-msg" ("string-nome" "void" "Retorna verdadeiro se a mensagem foi recebida desde o último quadro, e ajusta ela como a mensagem atual para chamadas subsequente ao (osc) para ler argumentos." " (cond \n     ((osc-msg \"/hello\")              ; if a the /hello message is recieved\n         (display (osc 1))(newline)))\t; print out the first argument\n")) ("osc" ("número-argumento" "void" "Retorna o argumento da mensagem osc atual." " (cond \n     ((osc-msg \"/hello\")              ; if a the /hello message is recieved\n         (display (osc 1))(newline)))\t; print out the first argument\n")) ("osc-destination" ("string-porta" "void" "Específica o destino para mensagens osc que estejam saindo. O nome da porta precisa ser específicado com todo o url e deve ser algo do tipo \"osc.udp://localhost:4444\"" " (osc-destination \"osc.udp:localhost:4444\")\n (osc-send \"/hello\" \"s\" (list \"boo!\"))  ; send a message to this destination\n")) ("osc-peek" ("" "msg-string" "Esta útil função retorna o nome, formato da string e número/string dos argumentos da última mensagem enviada como uma string - para debugar sua rede osc." " (display (osc-peek))(newline) \n")) ("osc-send" ("string-nome string-formato lista-argumento" "void" "Envia uma mensagem osc com o argumento da lista como dado osc. Somente suporta floats, ints e strings como dados. O formato-string deve ser composto de \"i\", \"f\" e \"s\", e deve bater com os tipos dados na lista. Isto poderia ser removido provavelmente, usando os tipos diretamente, mas fazendo isto desta forma permite que você diga o tipo da mensagem osc explicitamente." " (osc-destination \"osc.udp:localhost:4444\")\n (osc-send \"/hello\" \"sif\" (list \"boo!\" 3 42.3))  ; send a message to this destination\n"))))) ("scratchpad" ("O scratchpad é o editor fluxus e a janela gl." "" (("every-frame" ("função-callback" "void" "Ajusta uma função pra ser chamada todo o tempo em que o render está para desenhar um novo quadro." "   (define count 0)\n  \n   (define (myfunc)\n       (display count)(display \" frames have been rendered!\")\n       (newline)\n       (set! count (+ count 1)))\n  \n   (every-frame (myfunc)) \n  ")) ("clear" ("" "void" "Limpa o renderizador de todos os objetos e luzes. Limpa o sistema de física e re-inicializa a chamada de volta em every-frame. Geralmente uma boa coisa a fazer é colocar isto no ínicio dos scripts pra ter certeza que tudo esta limpo cada vez que você chamar a execução." "   (clear) ; sem isso a gente ia acumular um novo cubo toda vez que F5 fosse pressionado\n   (build-cube) \n  ")) ("start-framedump" ("string-nome string-tipo" "void" "Inicia a gravação de quadros no disco. Tipo pode ser um dos: \"tif\", \"jpg\" ou \"ppm\". Nomes dos arquivos são construidos junto com o número do quadro adicionado, prefixado em 5 zeros." "   (start-framedump \"frame\" \"jpg\")\n  ")) ("end-framedump" ("" "void" "Para a gravação de quadros para o disco." "   (end-framedump)\n  ")) ("help" ("string-função" "void" "Mostra informação de ajuda numa função do fluxus. Para rodar no repl principalmente." "   (help \"pop\")\n  ")) ("callback-override" ("" "void" "Permite que você substitua a chama de volta (callback) do quadro, para controlar o loop de renderização do fluxus de uma forma mais detalhada." "   (callback-override myfunc)\n  "))))) ("scratchpad-docs" ("O scratchpad é o editor fluxus e a janela gl." "" (("tophelp" ("" "" "" " Fluxus docs\n -----------\n Fluxus é uma máquina de renderização em tempo real para\n livecoding em Scheme.\n \n O scratchpad de Fluxus têm dois modos de operação, o console\n (você está usando ele agora) que permite que você entre comandos\n e veja o resultado imediatamente. O outro modo é o editor que se\n parece mais com um editor de texto normal -  você tem 9 áreas de\n trabalho( que permite que você edite mais de um script de uma\n vez) vá para eles usando ctrl-1 até ctrl-9 e venha de volta para\n o console através do atalho ctrl-0.                                   \n\n Mais tópicos de ajuda:\n (help \"keys\") para comandos do teclado que controlam o fluxus.\n (help \"console\") para mais ajuda sobre o console\n (help \"editor\") para mais ajuda no editor de livecoding\n (help \"camera\") para ajuda nos controles da câmera.\n (help \"functionname\") para encontrar mais informação sobre uma função.\n (help \"misc\") para informações miscelâneas sobre o fluxus\n")) ("keys" ("" "" "" " Teclas do Fluxus\n ----------------\n \n ctrl-f : modo Tela cheia.\n ctrl-w : modo Janela.\n ctrl-h : esconder/mostrar o editor.\n ctrl-l : Carregar um novo script (navegue com os cursores e enter)\n ctrl-s : Salve o script atual.\n ctrl-d : Salvar como - o script atual (abre um caixa com nome)\n ctrl-1 to 9 : Vai para a área de trabalho selecionada.\n ctrl-0 : Vai para o REPL. F3 : Reinicia a câmera.\n F5 : Executao texto selecionado, ou tudo se não tem nada selecionado.\n F9 : Randomisa a cor do texto (aka botão panico)\n")) ("console" ("" "" "" " Console do Fluxus (ou REPL) ---------------------------\n\n Se você pressionar ctrl e 0, ao invés de abrir outra área de\n trabalho para scripts você vai ser apresentado a um interpretador\n de Leitura EvaluaçãoImpressão e Loop, ou repl por\n diminutivo. Isto realmente é um interpretador interativo similar\n a uma linha de comando, onde você pode entrar código scheme para\n evaluação imediata. Estecódigo é evaluado no mesmo interpretador\n que os outros scripts, então você pode usar o repl para debug ou\n inspecionar variáveis globais e funções que elas definem. Esta janela é também onde todo os erros são impressos, junto com a\n janela de terminal que você iniciou o fluxus.\n")) ("editor" ("" "void" "" " EditorFluxus\n -------------\n\n Quando usar o scratchpad do fluxus, a idéia é que você só precise\n da única tela para construir os scripts, ou tocar ao vivo, F5 é a\n chave que roda o script quando vc estiver pronto. Selecionando\n texto (usando shift) e pressionando F5 vai executar somente o\n texto selecionado. Isto é uma mão na roda para re-evaluar funções\n sem rodar o script inteiro toda vez.\n\n Áreas de trabalho\n -----------------\n\n O editor de script permite que voce edite até 9 scripts\n simultaneamente usando diferentes áreas de trabalho. Para trocas\n de área, use ctrl+tecla de número. Somente um script pode rodar\n de uma vez no entanto, apertando F5 vai executar o script da área\n de trabalho ativa atualmente.\n")) ("misc" ("" "void" "EndFunctionDoc StartFunctionDoc-pt camera Retorna: void Descrição:" " Fluxus camera control\n ---------------------\n \n A câmera é controlada ao mover o mouse e pressionar os botões ao\n mesmo tempo.\n\n Botão esquerdo do mouse: Rotaciona\n Botão do meio do mouse: Movimenta\n Botão direito do mouse: Afasta/Aproxima(zoom)\n")) ("authors" ("" "void" "" "\n \"Computers are useless. They can only give you answers\". \n     Pablo Picasso (1881 - 1973).\n\n Authors\n -------\n Alex Norman \n Artem Baguinski\n Dan Bethell\n Dave Griffiths \n Glauber Alex Dias Prado\n Nik Gaffney\n James Tittle\n\n"))))))))